/*
 * IBM Technical Support Appliance Version 2 Release 8 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2021
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/IconFactory'
  , 'tsa/widgets/TablePanel'
  ],

  function( $
          , TV
          , AbstractHandler
          , Command
          , ShowCommand
          , ButtonFactory
          , ContentArea
          , FormPanel
          , IconFactory
          , TablePanel )
  {
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger( 'CredentialsHandler' );

    //-------------------------------------------------------------------------

    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters. */

    tsa.cmd.handlers.CredentialsHandler = function( parameters ) {
        if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call( this, parameters );

        // Initializing members...
        this.cachedScopeSets = {};
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.CredentialsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.CredentialsHandler.prototype ).constructor = tsa.cmd.handlers.CredentialsHandler;

    //-------------------------------------------------------------------------

    /**
     * The scope sets (mapped by PID).  List of scope sets used to populate restrict scope set pulldown (among
     * other things.  Should be refreshed each time the new or edit panel is displayed.
     * @type {Array.<Object>}
     */
    tsa.cmd.handlers.CredentialsHandler.prototype.cachedScopeSets = null;

    //-------------------------------------------------------------------------
    /**
     * Show main screen (list the credentials).
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._index = function() {
        this._list();
    };

    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** CREDENTIAL LIST SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Show the credentials list screen.
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._list = function() {
        /** @type {Object} */ var i18n = tsa.i18n;

        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title         : i18n.labelDiscoCredsHeading
                                                                                      , helpDoc       : 'sshelp_discovery_credentials_list_panel.html'
                                                                                      , showBackToTop : true } );
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
        /** @type {tsa.cmd.Command}         */ var getCredCommand  = null;

        tsa.setLayout( 'wide' );

        contentArea.appendMessage( { text : i18n.labelDiscoCredsCaption } );
        contentArea.appendMessage( { text : i18n.labelDiscoCredsCaption2 } );

        var addCredClicked = $.proxy( function( event ) {
            /** @type {tsa.cmd.ShowCommand} */ var showCmd = new ShowCommand( { action : 'credentials/new' } );

            showCmd.execute();
        }
        , this );

        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                , label   : i18n.buttonDiscoCredsAdd
                                                                , onclick : addCredClicked } ) );

        this.show( contentArea.getDomNode() );
        tsa.setLayout( 'wide' );

        // Retrieving the credentials from server...
        getCredCommand = new Command( { action              : 'credentials/getCredentials'
                                      , clearSystemMessages : false } );

        // Credentials will be used to populate the credentials table...
        $( getCredCommand ).on( 'executionSucceeded', $.proxy( function( event ) {
            /** @type {Array.<Object>}         */ var credentials     = event.response.payload.credentials;
            /** @type {tsa.widgets.TablePanel} */ var tablePanel      = null;
            /** @type {Object}                 */ var tablePanelProps = { title : i18n.labelDiscoCredsCredentialsHeading };

            if (TV.typeIsObject( typeof credentials ) && TV.isInstanceOf( credentials, Array ) && ( credentials.length > 0 )) {
                /** @type {Array.<Object>} */ var columnData       = [];
                var widthColumn = ["25%", "15%", "15%", "10%", "10%", "15%", "10%"];
                if (tsa.isUserLanguageCode(tsa.GERMAN_LANG_CODE)) {
                    widthColumn = ["2%", "2%", "5.5%", "4%", "7%", "8%", "3%"];
                    tablePanelProps.tableLayout = "fixed"; 
                }
                columnData.push( { id : 'name'          , label : i18n.labelDiscoCredsName                , width : widthColumn[0] , sortable : false }
                               , { id : 'type'          , label : i18n.labelDiscoCredsType                , width : widthColumn[1] , sortable : false }
                               , { id : 'authType'      , label : i18n.labelDiscoCredsAuthenticationType  , width : widthColumn[2] , sortable : false }
                               , { id : 'userName'      , label : i18n.labelDiscoCredsUserName            , width : widthColumn[3] , sortable : false }
                               , { id : 'lastPwdChange' , label : i18n.labelDiscoCredsPasswordChangedDate , width : widthColumn[4] , sortable : false }
                               , { id : 'restrictions'  , label : i18n.labelDiscoCredsRestrictions        , width : widthColumn[5] , sortable : false }
                               , { id : 'actions'       , label : i18n.labelActionsColumn                 , width : widthColumn[6] , sortable : false } );
                tablePanelProps.columns = columnData;

            } else {
                tablePanelProps.emptyMsg = i18n.labelDiscoCredsNoCreds;
            }

            tablePanel = new TablePanel( tablePanelProps );

            // Iterating credentials, building rows...
            $.each( credentials, function( credIndex, cred ) {
                /** @type {Array.<Element>} */ var elements     = [];

                /** @type {Element} */ var nameSpan     = document.createElement( 'span' );
                /** @type {Element} */ var nameAnchor   = document.createElement( 'a' );
                /** @type {Element} */ var scopeSetSpan = document.createElement( 'span' );
                /** @type {Element} */ var actionsSpan  = document.createElement( 'span' );
                /** @type {Element} */ var editIcon     = IconFactory.buildActionIcon( { type : 'edit'     , caption : tsa.bind(i18n.labelButtonEdit,cred.name)} );
                /** @type {Element} */ var deleteIcon   = IconFactory.buildActionIcon( { type : 'delete'   , caption : tsa.bind(i18n.labelButtonDelete,cred.name)} );
                /** @type {Element} */ var moveDownIcon = IconFactory.buildActionIcon( { type : 'movedown' , caption : tsa.bind(i18n.labelDiscoCredsMoveDown,cred.name) } );
                /** @type {Element} */ var moveUpIcon   = IconFactory.buildActionIcon( { type : 'moveup'   , caption : tsa.bind(i18n.labelDiscoCredsMoveUp,cred.name) } );
                /** @type {boolean} */ var isFirst      = ( credIndex === 0 );
                /** @type {boolean} */ var isLast       = ( credIndex === ( credentials.length - 1 ) );

                //
                // Column 1 - Name:
                //
                nameAnchor.href = '#';
                $( nameAnchor ).addClass( 'ibm-feature-link' );
                $( nameAnchor ).css( { 'font-weight' : 'bold' } );
                $( nameAnchor ).text( cred.name );
                $( nameAnchor ).on( 'click', function( event ) {
                    tsa.cmd.handlers.CredentialsHandler.showCredentialClicked( cred.pid );

                    try {
                        event.stopPropagation();
                    } catch ( error ) { /* Ignore */ }

                    return false;
                } );

                $( nameSpan ).append( nameAnchor );
                $( nameSpan ).append( document.createTextNode( tsa.isDebugModeOn() ? ' (' + cred.properties.order + ')' : '' ) );
                elements.push( nameSpan );

                //
                // Column 2 - Class label:
                //
                elements.push( tsa.cmd.handlers.CredentialsHandler.i18nCredType(cred));

                //
                // Column 3 - password type:
                //
                elements.push( tsa.cmd.handlers.CredentialsHandler.i18nAuthType(cred));

                //
                // Column 4 - User name:
                //
                elements.push( ( cred.userName !== null ) ? cred.userName : '' );

                //
                // Column 5 - Password last changed:
                //
                elements.push( ( cred.passwordLastChanged !== null ) ? cred.passwordLastChanged : i18n.labelDiscoCredsPasswordChangeNotAvailable );

                //
                // Column 6 - Scope Set:
                $( scopeSetSpan ).text( ( cred.scopeSetName !== null ) ? cred.scopeSetName : '' );
                elements.push( scopeSetSpan );

                //
                // Column 7 - Actions:
                //
                $( actionsSpan ).append( editIcon   );
                $( actionsSpan ).append( deleteIcon );

                if (!isLast ) $( actionsSpan ).append( moveDownIcon );
                if (!isFirst) $( actionsSpan ).append( moveUpIcon   );

                $.each( [ editIcon, deleteIcon, moveDownIcon, moveUpIcon ], function( iconIndex, icon ) {
                    $( icon ).css( { 'cursor'       : 'pointer'
                                   , 'margin-right' : '0.5em' } );
                } );

                $( editIcon     ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.editCredentialClicked( cred.pid );              } );
                $( deleteIcon   ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.deleteCredentialClicked( cred.name, cred.pid ); } );
                $( moveDownIcon ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.moveCredentialClicked( cred.pid,  1 );          } );
                $( moveUpIcon   ).on( 'click', function( event ) { tsa.cmd.handlers.CredentialsHandler.moveCredentialClicked( cred.pid, -1 );          } );

                elements.push( actionsSpan );

                // Append the row...
                tablePanel.appendRow( elements );
            } );

            contentArea.append( tablePanel.getDomNode() );
            contentArea.append( buttonContainer );
        }
        , this ) );

        getCredCommand.execute();
    };

    //-------------------------------------------------------------------------

    /**
     * Show credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsHandler.showCredentialClicked = function( credentialPID ) {
        ( new ShowCommand( { action : 'credentials/show/' + credentialPID } ) ).execute();
    };

    //-------------------------------------------------------------------------

    /**
     * Edit credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsHandler.editCredentialClicked = function( credentialPID ) {
        ( new ShowCommand( { action : 'credentials/edit/' + credentialPID } ) ).execute();
    };

    //-------------------------------------------------------------------------

    /**
     * Delete credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialsHandler.deleteCredentialClicked = function( credentialName, credentialPID ) {
        if (TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 )) {
            if (window.confirm( tsa.bind(tsa.i18n.labelDeleteConfirmation, credentialName ) )) {
                /** @type {tsa.cmd.Command} */ var deleteCmd = new Command( { action     : 'credentials/deleteCredential'
                                                                            , parameters : { 'pid' : credentialPID } } );

                $( deleteCmd ).on( 'executionSucceeded', function( event ) {
                    ( new ShowCommand( { action              : 'credentials/index'
                                       , clearSystemMessages : false } ) ).execute();
                } );

                deleteCmd.execute();
            }

        } else {
            logger.warn( 'Invalid credential PID. Deletion request did not proceed.' );
        }
    };

    //-------------------------------------------------------------------------

    /**
     * Move credential - event handler.
     * @param {string} credentialPID The credential PID.
     * @param {number} offset        Negative to move up, positive to move down.
     */
    tsa.cmd.handlers.CredentialsHandler.moveCredentialClicked = function( credentialPID, offset ) {
        if (TV.typeIsString( typeof credentialPID ) && ( credentialPID.length > 0 )) {
            if (TV.typeIsNumber( typeof offset ) && !isNaN( offset ) && ( offset !== 0 ))  {
                /** @type {tsa.cmd.Command} */ var moveCmd = new Command( { action     : 'credentials/ajaxMoveCredential'
                                                                          , parameters : { 'pid'    : credentialPID
                                                                                         , 'offset' : offset } } );

                $( moveCmd ).on( 'executionSucceeded', function( event ) {
                    ( new ShowCommand( { action              : 'credentials/index'
                                       , clearSystemMessages : false } ) ).execute();
                } );

                moveCmd.execute();

            } else {
                logger.error( 'Invalid offset. Movement request did not proceed.' );
            }

        } else {
            logger.error( 'Invalid credential PID. Movement request did not proceed.' );
        }
    };

    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** SHOW CREDENTIAL SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Show the credential details.
     * The credential PID is provided as the entity ID (last element in the URL)
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._show = function() {
        /** @type {Object} */ var i18n = tsa.i18n;

        /** @type {string} */ var pid = this.getParameter( 'entityId' );

        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelDiscoCredsHeading
                                                                                      , helpDoc : 'sshelp_discovery_credential_show_panel.html' } );
        /** @type {tsa.widgets.TablePanel}  */ var generalTable    = new TablePanel( { title   : i18n.labelDiscoCredsGeneralHeading
                                                                                     , columns : [ { id: 'name'  , label: '' , width: '30%' , sortable: false }
                                                                                                 , { id: 'value' , label: '' , width: '70%' , sortable: false } ] } );
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();

        var getCredentialCmd = new Command( { action              : 'credentials/getCredential'
                                            , parameters          : { 'pid' : pid }
                                            , clearSystemMessages : false } );

        $( getCredentialCmd ).on( 'executionSucceeded', function( event ) {
            /** @type {Object} */ var credential = event.response.payload.credential;

            if (TV.typeIsObject( typeof credential ) && ( credential !== null )) {
                /** @type {Element} */ var scopeSetSpan      = document.createElement( 'span' );
                /** @type {?string} */ var userName          = credential.userName;
                /** @type {?string} */ var scopeSetName      = credential.scopeSetName;
                /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof scopeSetName ) && ( scopeSetName.length > 0 ) );
                /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
                /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );

                if( userName     === null ) userName     = '';
                if( scopeSetName === null ) scopeSetName = '';

                $( scopeSetSpan ).text( scopeSetName );

                generalTable.appendRow( [ i18n.labelNameField          , credential.name                                              ] );
                generalTable.appendRow( [ i18n.labelTypeField          , tsa.cmd.handlers.CredentialsHandler.i18nCredType(credential) ] );
                generalTable.appendRow( [ i18n.labelUserNameFieldColon , userName                                                     ] );
                generalTable.appendRow( [ i18n.labelScopeSetField      , scopeSetSpan                                                 ] );

                // Show authentication type only for those credential types that support it.
                if (TV.typeIsObject( typeof credential.properties ) && ( credential.properties !== null )) {
                    for( /** @type {string} */ var propertyName in credential.properties ) {
                        /** @type {string} */ var propertyValue = credential.properties[ propertyName ];

                        if (propertyName === 'authtype') {
                            if (propertyValue === 'authType_password') {
                                generalTable.appendRow( [ i18n.labelAuthenticationTypeField , tsa.i18n.labelCredentialsAuthenticationTypePassword ] );
                            } else {
                                generalTable.appendRow( [ i18n.labelAuthenticationTypeField , tsa.i18n.labelCredentialsAuthenticationTypePkiScopedFull ] );
                            }
                        }
                    }
                }

                // If TSA web UI is in debug mode, show a few more fields.
                if (tsa.isDebugModeOn()) {
                    generalTable.appendRow( [ i18n.labelDiscoCredsGeneralPidDebug   , credential.pid           ] );
                    generalTable.appendRow( [ i18n.labelDiscoCredsGeneralClassDebug , credential.authClassName ] );
                }
            }

            $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'back'
                                                                    , grayout : true
                                                                    , label   : i18n.buttonGoBack
                                                                    , onclick : function( event ) { ( new ShowCommand( { action : 'credentials/index' } ) ).execute(); } } ) );
            $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
            $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                    , label   : i18n.buttonDiscoCredsEdit
                                                                    , onclick : function( event ) { tsa.cmd.handlers.CredentialsHandler.editCredentialClicked( credential.pid ); } } ) );

            contentArea.append( generalTable.getDomNode() );
            contentArea.append( buttonContainer );
        } );

        getCredentialCmd.execute();

        this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** ADD CREDENTIAL SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Add a new credential.
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._new = function() {
        /** @type {Object} */ var i18n = tsa.i18n;

        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title   : i18n.labelDiscoCredsNewCredsHeading
                                                                                      , helpDoc : 'sshelp_discovery_credential_new_panel.html' } );
        /** @type {tsa.widgets.FormPanel}   */ var credForm        = this.createCredentialForm();
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();

        credForm.setActionAttribute( 'credentials/postCredential' );

        // Buttons...
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                , label   : i18n.buttonSave
                                                                , onclick : function( event ) { credForm.putHiddenInput( 'testCredential', null );
                                                                                                credForm.submit(); } } ) );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                                , label   : i18n.buttonCancel
                                                                , onclick : function( event ) { ( new ShowCommand( { action : 'credentials/index' } ) ).execute(); } } ) );

        // Appending content...
        contentArea.append( credForm.getDomNode() );
        contentArea.append( buttonContainer );
        this.show( contentArea.getDomNode() );

        // What happens when creation succeeds? Print success message, then show the credentials list.
        $( credForm ).on( 'executionSucceeded', function( event ) {
            if (credForm.getHiddenInput("testCredential") == null) {
                /** @type {string} */ var credentialName = $( credForm.getDomNode() ).find( '[name=\'name\']' ).val();
                tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoCredsCreated, credentialName ) } );
                ( new ShowCommand( { action              : 'credentials/index'
                                   , clearSystemMessages : false } ) ).execute();
            }
        } );

        $( credForm ).on( 'executionFailed', function( event ) {
            if ( TV.typeIsObject( typeof event.response.payload.errors ) ) {
                for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                    credForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                }
                delete event.response.payload.errors;
            }
        } );
    };


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** EDIT CREDENTIAL SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Edit an existing credential.
     */
    tsa.cmd.handlers.CredentialsHandler.prototype._edit = function() {
        /** @type {Object}*/ var i18n = tsa.i18n;

        /** @type {String}*/ var credentialId = this.getParameter( 'entityId' );

        /** @type {tsa.widgets.ContentArea} */ var contentArea     = new ContentArea( { title         : i18n.labelDiscoEditCredsHeading
                                                                                      , helpDoc       : 'sshelp_discovery_credential_edit_panel.html' } );
        /** @type {tsa.widgets.FormPanel}   */ var credForm        = null;
        /** @type {Element}                 */ var buttonContainer = ButtonFactory.buildButtonContainer();
        /** @type {tsa.cmd.Command}         */ var getCredCmd      = new Command( { action              : 'credentials/getCredential'
                                                                                  , parameters          : { 'pid' : credentialId }
                                                                                  , clearSystemMessages : false } );

        $( getCredCmd ).on( 'executionSucceeded', $.proxy( function( event ) {
            credForm = this.createCredentialForm( event.response.payload.credential );
            credForm.setActionAttribute( 'credentials/putCredential' );

            // What happens when edition succeeds? Print success message, then show the credentials list.
            $( credForm ).on( 'executionSucceeded', function( event ) {
                if (credForm.getHiddenInput("testCredential") == null) {
                    /** @type {string} */ var credentialName = $( credForm.getDomNode() ).find( '[name=\'name\']' ).val();

                    tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoCredsUpdated, credentialName ) } );

                    ( new ShowCommand( { action              : 'credentials/index'
                                       , clearSystemMessages : false } ) ).execute();
                }
            } );

            // Appending content...
            contentArea.append( credForm.getDomNode() );
            contentArea.append( buttonContainer );
        }
        , this ) );

        // Buttons...
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'arrow'
                                                                , label   : i18n.buttonSave
                                                                , onclick : function( event ) { credForm.putHiddenInput( 'testCredential', null );
                                                                                                credForm.submit(); } } ) );
        $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
        $( buttonContainer ).append( ButtonFactory.buildButton( { type    : 'cancel'
                                                                , label   : i18n.buttonCancel
                                                                , onclick : function( event ) { ( new ShowCommand( { action : 'credentials/index' } ) ).execute(); } } ) );

        // Show content area now...
        this.show( contentArea.getDomNode() );

        // Get the credential...
        getCredCmd.execute();
    };

    //-------------------------------------------------------------------------

    /**
     * Build the creation/edition form.
     * @param {Object=} credential The credential data to initialize the form. Optional.
     * @return {tsa.widgets.FormPanel} The resulting form panel.
     * @private
     */
    tsa.cmd.handlers.CredentialsHandler.prototype.createCredentialForm = function( credential ) {
        /** @type {Object} */ var i18n = tsa.i18n;

        /** @type {boolean} */ var isEditMode = TV.typeIsObject( typeof credential ) && ( credential !== null );

        /** @type {tsa.widgets.FormPanel} */ var generalForm     = null;
        /** @type {tsa.widgets.FormPanel} */ var nameForm        = new FormPanel( { title       : i18n.labelDiscoCredsNameHeading
                                                                                  , description : i18n.labelDiscoCredsNameDesc } );
        /** @type {tsa.widgets.FormPanel} */ var typeForm        = new FormPanel( { title       : i18n.labelDiscoCredsSelectHeading
                                                                                  , description : i18n.labelDiscoCredsSelectDesc } );
        /** @type {tsa.widgets.FormPanel} */ var restrictionForm = new FormPanel( { title       : i18n.labelDiscoCredsScopeRestrictHeading
                                                                                  , description : i18n.labelDiscoCredsScopeRestrictDesc } );
        /** @type {tsa.widgets.FormPanel} */ var scopeSetForm    = new FormPanel( { title       : i18n.labelDiscoCredsScopeSetHeading
                                                                                  , description : i18n.labelDiscoCredsScopeSetDesc } );
        /** @type {tsa.widgets.FormPanel} */ var testForm        = new FormPanel( { title       : i18n.labelDiscoCredsTestHeading
                                                                                  , description : i18n.labelDiscoCredsTestDesc } );
        /** @type {tsa.widgets.FormPanel} */ var firstAccessForm = null;
        /** @type {tsa.widgets.FormPanel} */ var lastAccessForm  = null;
        /** @type {tsa.widgets.FormPanel} */ var firstForm       = nameForm;

        /** @type {tsa.cmd.Command} */ var getScopeSetsCmd = new Command( { action              : 'scopesets/getScopeSets'
                                                                          , parameters          : { 'withCredentials' : 'true' }
                                                                          , clearSystemMessages : false } );

        /** @type {Object.<string,tsa.widgets.FormPanel>} */ var accessForms =
        { HostAuth            : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsHostAuthEnterDesc               } )
        , WindowsHostAuth     : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWindowsHostAuthEnterDesc        } )
      //, JBossAuth           : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsJBossAuthEnterDesc              } )
      //, OracleAppAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsOracleAppAuthEnterDesc          } )
      //, WeblogicAuth        : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWeblogicAuthEnterDesc           } )
      //, WebSphereAuth1      : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWebSphereAuthEnterDesc          } )
      //, WebSphereAuth2      : new FormPanel( { title: i18n.labelDiscoCredsWebsphereKeyStoreHeading   , description: i18n.labelDiscoCredsWebSphereAuthKeyStoreEditDesc   } )
      //, WebSphereAuth3      : new FormPanel( { title: i18n.labelDiscoCredsWebsphereTrustStoreHeading , description: i18n.labelDiscoCredsWebSphereAuthTrustStoreEditDesc } )
      //, WasSshAuth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsWasSshAuthEnterDesc             } )
      //, Db2Auth             : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsDb2AuthEnterDesc                } )
      //, OracleAuth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsOracleAuthEnterDesc             } )
      //, SybaseAuth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSybaseAuthEnterDesc             } )
      //, SqlServerAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSqlServerAuthEnterDesc          } )
      //, DominoServerAuth    : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsDominoServerAuthEnterDesc       } )
      //, MQServerAuth        : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsMQServerAuthEnterDesc           } )
      //, ExchangeServerAuth  : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsExchangeServerAuthEnterDesc     } )
        , SnmpAuth            : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSnmpAuthEnterDesc               } )
        , SnmpV3Auth          : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSnmpV3AuthEnterDesc             } )
      //, CCMSServerAuth      : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCCMSServerAuthEnterDesc         } )
        , CiscoAuth           : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCiscoAuthEnterDesc              } )
        , CiscoWorksAuth      : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCiscoWorksAuthEnterDesc         } )
      //, LdapAuth            : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsLdapAuthEnterDesc               } )
      //, SLDServerAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSLDServerAuthEnterDesc          } )
      //, CSMServerAuth       : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsCSMServerAuthEnterDesc          } )
      //, VeritasClusterAuth  : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsVeritasClusterAuthEnterDesc     } )
      //, ITMScopeAuth        : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsITMScopeAuthEnterDesc           } )
      //, SysImagerServerAuth : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsSysImagerServerAuthEnterDesc    } )
      //, VCAuth              : new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading         , description: i18n.labelDiscoCredsVCAuthEnterDesc                 } )
        };

        // Linking access forms...
        for( /** @type {string} */ var name in accessForms ) {
            if (lastAccessForm !== null) {
                lastAccessForm.linkTo( accessForms[ name ] );
            }

            if (firstAccessForm === null) {
                firstAccessForm = accessForms[ name ];
            }

            lastAccessForm = accessForms[ name ];
        }

        // Linking all forms together...
        nameForm        .linkTo( typeForm        );
        typeForm        .linkTo( firstAccessForm );
        lastAccessForm  .linkTo( restrictionForm );
        restrictionForm .linkTo( scopeSetForm    );
        scopeSetForm    .linkTo( testForm        );

        // Fields (common):
        /** @type {Element} */ var nameField     = $( '<input type="text" id="name" name="name" size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var typeField     = $( '<select name="CredentialType" id="CredentialType" >'
                                                    +   '<option value="HostAuth" selected="selected"' + '>' + i18n.labelDiscoCredsTypeHostAuth        + '</option>'
                                                    +   '<option value="WindowsHostAuth"'              + '>' + i18n.labelDiscoCredsTypeWindowsHostAuth + '</option>'
                                                    +   '<option value="SnmpAuth"'                     + '>' + i18n.labelDiscoCredsTypeSnmpAuth        + '</option>'
                                                    +   '<option value="SnmpV3Auth"'                   + '>' + i18n.labelDiscoCredsTypeSnmpV3Auth      + '</option>'
                                                    +   '<option value="CiscoAuth"'                    + '>' + i18n.labelDiscoCredsTypeCiscoAuth       + '</option>'
                                                    +   '<option value="CiscoWorksAuth"'               + '>' + i18n.labelDiscoCredsTypeCiscoWorksAuth  + '</option>'
                                                    + '</select>' ).get(0);

        // Radio buttons for selecting restriction type
        /** @type {Element} */ var restrictionAllRadio   = document.createElement( 'input' );
        /** @type {Element} */ var restrictionScopeRadio = document.createElement( 'input' );

        /** @type {Element} */ var scopeSetField = $( '<select name="scopename" id="scopename"></select>' ).get(0);
        /** @type {Element} */ var testhostField = $( '<input type="text" id="ipAddress" name="ipAddress" size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var testButton    = ButtonFactory.buildButton( { type: 'arrow', label: i18n.labelDiscoCredsTestButton } );
        /** @type {Element} */ var testButtonCon = ButtonFactory.buildButtonContainer();

        $( testButtonCon ).append( testButton );

        nameForm.appendFieldRow( { label: i18n.labelDiscoCredsName       , required: true , element: nameField } );
        typeForm.appendFieldRow( { label: i18n.labelDiscoCredsSelectType , required: true , element: typeField , smallLabel : tsa.isUserLanguageCode(tsa.GERMAN_LANG_CODE) } );

        // Authentication type radio buttons...
        restrictionScopeRadio.type  = 'radio';
        restrictionScopeRadio.name  = 'RestrictionType';
        restrictionScopeRadio.value = 'limit';

        restrictionAllRadio.type  = 'radio';
        restrictionAllRadio.name  = 'RestrictionType';
        restrictionAllRadio.value = 'all';

        if (isEditMode) {
              // Determine scope restriction (if one exists) and select the right radio button.
            if (credential.scopeSet === null) {
                restrictionAllRadio.checked = true;
                scopeSetForm.hide();
            } else {
                restrictionScopeRadio.checked = true;
                scopeSetForm.show();
            }
        } else {
            // On create credential, restricted is always the default.
            restrictionScopeRadio.checked = true;
            scopeSetForm.show();
        }

        restrictionForm.appendFieldRow( {
            label    : i18n.labelDiscoCredsScopeRestrict
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                             /** @type {Element} */ var span                  = document.createElement( 'span' );
                             /** @type {Element} */ var restrictionScopeLabel = document.createElement( 'label' );
                             /** @type {Element} */ var restrictionAllLabel   = document.createElement( 'label' );

                             $( restrictionScopeLabel ).append( restrictionScopeRadio, document.createTextNode( i18n.labelDiscoCredsScopeRestrictLimit ) );
                             $( restrictionAllLabel   ).append( restrictionAllRadio,   document.createTextNode( i18n.labelDiscoCredsScopeRestrictAll  ) );
                             $( span                  ).append( restrictionScopeLabel, document.createElement( 'br' ), restrictionAllLabel );
                             $( span                  ).addClass( 'ibm-input-group' );
                             $( span                  ).attr( 'role', 'radiogroup' );
                             $( span                  ).attr( 'aria-label', 'Scope radio' ); 

                         return span;
                       } )()
        } );

        scopeSetForm.appendFieldRow( { label: i18n.labelDiscoCredsScopeSetName  , required: true  , element: scopeSetField } );
        testForm    .appendFieldRow( { label: i18n.labelDiscoCredsTestIP        , required: false , element: testhostField , smallLabel : tsa.isUserLanguageCode(tsa.JAPAN_LANG_CODE) } );
        testForm    .appendHR();
        testForm    .appendJointRow( testButtonCon );

        if (isEditMode) {
            /** @type {Element} */ var nameSpan = document.createElement( 'span' );
            /** @type {Element} */ var typeSpan = document.createElement( 'span' );

            generalForm = new FormPanel( { title: i18n.labelDiscoCredsGeneralHeading } );

            $( nameSpan ).css( { 'font-weight' : 'bold' } );
            $( nameSpan ).text( credential.name           );
            $( typeSpan ).text( tsa.cmd.handlers.CredentialsHandler.i18nCredType(credential) );

            generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralName, element: nameSpan } );
            generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralType, element: typeSpan } );

            generalForm.linkTo( nameForm );
            firstForm = generalForm;

            firstForm.putHiddenInput( 'pid'            , credential.pid                 );
            firstForm.putHiddenInput( 'name'           , credential.name                );
            firstForm.putHiddenInput( 'CredentialType' , credential.authClassSimpleName );

            $( nameField ).val( credential.name                );
            $( typeField ).val( credential.authClassSimpleName );

            nameForm.hide();
            typeForm.hide();
        }

        /*
         * Fields for HostAuth:
         */

        // Radio buttons for selecting authentication type.
        /** @type {Element} */ var hostAuthTypePasswordRadio = document.createElement( 'input' );
        /** @type {Element} */ var hostAuthTypePkiRadio      = document.createElement( 'input' );

        /** @type {Element} */ var hostAuthUserName      = $( '<input type="text"     id="HostAuth_username"         name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var hostAuthPassword1     = $( '<input type="password" id="HostAuth_password"         name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var hostAuthPassword2     = $( '<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var hostAuthUploadKey     = $( '<input type="file"     id="HostAuth_sshKey"           name="HostAuth_sshKey" />' ).get(0);
        /** @type {Element} */ var hostAuthUploadKeyType = document.createElement( 'p' );
        /** @type {Element} */ var hostAuthUploadKeyWarn = document.createElement( 'p' );
        /** @type {Element} */ var hostAuthUploadKeyDiv  = document.createElement( 'div' );

        $( hostAuthUploadKeyDiv  ).css( { 'font-size'        : '1.33em' } );
        $( hostAuthUploadKeyWarn ).css( { 'margin-top'       : '0.80em'
                                        , 'border'           : '1px solid #c0c000'
                                        , 'border-radius'    : '5px'
                                        , 'background-color' : '#ffffc0' } );

        $( hostAuthUploadKeyDiv ).append( hostAuthUploadKeyType );
        $( hostAuthUploadKeyDiv ).append( hostAuthUploadKey     );
        $( hostAuthUploadKeyDiv ).append( hostAuthUploadKeyWarn );

        // Authentication type radio buttons...
        hostAuthTypePasswordRadio.type  = 'radio';
        hostAuthTypePasswordRadio.name  = 'HostAuth_authtype';
        hostAuthTypePasswordRadio.value = 'authType_password';

        hostAuthTypePkiRadio.type  = 'radio';
        hostAuthTypePkiRadio.name  = 'HostAuth_authtype';
        hostAuthTypePkiRadio.value = 'authType_pki';

        if (isEditMode) {
            // Determine the existing credential authentication type and select the right radio button.
            var authType = credential.properties.authtype;
            if (authType === "authType_pki") {
                hostAuthTypePkiRadio.checked = true;
            } else {
                hostAuthTypePasswordRadio.checked = true;
            }
        } else {
            // On create credential, password is always the default.
            hostAuthTypePasswordRadio.checked = true;
        }

        accessForms.HostAuth.appendFieldRow( {
            label    : i18n.labelAuthenticationTypeField
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                             /** @type {Element} */ var span          = document.createElement( 'span' );
                             /** @type {Element} */ var passwordLabel = document.createElement( 'label' );
                             /** @type {Element} */ var pkiLabel      = document.createElement( 'label' );

                             $( passwordLabel ).append( hostAuthTypePasswordRadio, document.createTextNode( i18n.labelCredentialsAuthenticationTypePassword      ) );
                             $( pkiLabel      ).append( hostAuthTypePkiRadio,      document.createTextNode( i18n.labelCredentialsAuthenticationTypePkiScopedFull ) );
                             $( span          ).append( passwordLabel, document.createElement( 'br' ), pkiLabel );
                             $( span          ).addClass( 'ibm-input-group' );
                             $( span          ).attr( 'role', 'radiogroup' );
                             $( span                  ).attr( 'aria-label', 'Password radio' ); 

                             return span;
                         } )()
        } );

        accessForms.HostAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser       , required: true , element: hostAuthUserName     } );
        accessForms.HostAuth.appendFieldRow( { label: i18n.labelPasswordField        , required: true , element: hostAuthPassword1    } );
        accessForms.HostAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField , required: true , element: hostAuthPassword2    } );
        accessForms.HostAuth.appendFieldRow( { label: i18n.labelDiscoCredsSSHKey     , required: true , element: hostAuthUploadKeyDiv } );

        /*
         * Fields for WindowsHostAuth:
         */
        /** @type {Element} */ var windowsHostAuthUserName  = $( '<input type="text"     id="WindowsHostAuth_username"         name="WindowsHostAuth_username"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var windowsHostAuthPassword1 = $( '<input type="password" id="WindowsHostAuth_password"         name="WindowsHostAuth_password"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var windowsHostAuthPassword2 = $( '<input type="password" id="WindowsHostAuth_password_confirm" name="WindowsHostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);

        accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser       , required: true , element: windowsHostAuthUserName  } );
        accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelPasswordField        , required: true , element: windowsHostAuthPassword1 } );
        accessForms.WindowsHostAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField , required: true , element: windowsHostAuthPassword2 } );

        // Common value initialization for fields of HostAuth and WindowsHostAuth:
        if (isEditMode) {
            /** @type {boolean} */ var isHostAuth    = ( credential.authClassSimpleName === 'HostAuth'        );
            /** @type {boolean} */ var isWinHostAuth = ( credential.authClassSimpleName === 'WindowsHostAuth' );

            if (isHostAuth || isWinHostAuth) {
                /** @type {Element} */ var usernameField     = ( isWinHostAuth ? windowsHostAuthUserName : hostAuthUserName );
                /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof credential.scopeSetName ) &&
                                                               ( credential.scopeSetName.length > 0 ) );
                /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
                /** @type {string}  */ var authType          = credential.properties.authtype;
                /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );

                $( usernameField ).val( credential.properties.username );
            }
        }

        // Fields for SnmpAuth:
        /** @type {Element} */ var snmpAuthCommString1 = $( '<input type="password" name="SnmpAuth_communitystring" id="SnmpAuth_communitystring"        size="50" maxlength="250" required/>' ).get(0);
        /** @type {Element} */ var snmpAuthCommString2 = $( '<input type="password" name="SnmpAuth_communitystring_confirm" id="SnmpAuth_communitystring_confirm" size="50" maxlength="250" required/>' ).get(0);

        accessForms.SnmpAuth.appendFieldRow( { label: i18n.labelDiscoCredsCommunity1 , required: true  , element: snmpAuthCommString1 } );
        accessForms.SnmpAuth.appendFieldRow( { label: i18n.labelDiscoCredsCommunity2 , required: true  , element: snmpAuthCommString2 , smallLabel : tsa.isUserLanguageCode(tsa.JAPAN_LANG_CODE) } );

        // Fields for SnmpV3Auth:
        /** @type {Element} */ var snmpV3AuthUserName  = $( '<input type="text"     name="SnmpV3Auth_username" id="SnmpV3Auth_username"            size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var snmpV3AuthPassword1 = $( '<input type="password" name="SnmpV3Auth_password" id="SnmpV3Auth_password"            size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var snmpV3AuthPassword2 = $( '<input type="password" name="SnmpV3Auth_password_confirm"  id="SnmpV3Auth_password_confirm"    size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var snmpV3AuthPrivPwd1  = $( '<input type="password" name="SnmpV3Auth_privpassword"  id="SnmpV3Auth_privpassword"       size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var snmpV3AuthPrivPwd2  = $( '<input type="password" name="SnmpV3Auth_privpassword_confirm"  id="SnmpV3Auth_privpassword_confirm" size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var snmpV3AuthAuthProto = $( '<select name="SnmpV3Auth_authprotocol" id="SnmpV3Auth_authprotocol" >'
                                                          +   '<option value="None"' + '>' + i18n.labelDiscoCredsAuthProtoNone + '</option>'
                                                          +   '<option value="MD5"'  + '>' + i18n.labelDiscoCredsAuthProtoMd5  + '</option>'
                                                          +   '<option value="SHA"'  + '>' + i18n.labelDiscoCredsAuthProtoSha  + '</option>'
                                                          + '</select>' ).get(0);

        if (isEditMode) {
            $( snmpV3AuthUserName  ).val( credential.properties.username     );
            $( snmpV3AuthAuthProto ).val( credential.properties.authprotocol );
            if (credential.properties.authprotocol === 'MD5' || credential.properties.authprotocol === 'SHA') {
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsUser          , required: true  , element: snmpV3AuthUserName  } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelPasswordField           , required: true , element: snmpV3AuthPassword1 } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelConfirmPasswordField    , required: true , element: snmpV3AuthPassword2 } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw1 , required: true , element: snmpV3AuthPrivPwd1  } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw2 , required: true , element: snmpV3AuthPrivPwd2  } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsAuthProto     , required: false , element: snmpV3AuthAuthProto } );

            } else {
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsUser          , required: true  , element: snmpV3AuthUserName  } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelPasswordField           , required: false , element: snmpV3AuthPassword1 } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelConfirmPasswordField    , required: false , element: snmpV3AuthPassword2 } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw1 , required: false , element: snmpV3AuthPrivPwd1  } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw2 , required: false , element: snmpV3AuthPrivPwd2  } );
                accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsAuthProto     , required: false , element: snmpV3AuthAuthProto } );
            }

        } else {
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsUser          , required: true  , element: snmpV3AuthUserName  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelPasswordField           , required: false , element: snmpV3AuthPassword1 } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelConfirmPasswordField    , required: false , element: snmpV3AuthPassword2 } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw1 , required: false , element: snmpV3AuthPrivPwd1  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsPrivatePassw2 , required: false , element: snmpV3AuthPrivPwd2  } );
            accessForms.SnmpV3Auth.appendFieldRow( { label: i18n.labelDiscoCredsAuthProto     , required: false , element: snmpV3AuthAuthProto } );
        }

        /*
         * Fields for CiscoAuth:
         */
        /** @type {Element} */ var ciscoAuthUserName   = $( '<input type="text"     name="CiscoAuth_username"   id="CiscoAuth_username"            size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var ciscoAuthPassword1  = $( '<input type="password" name="CiscoAuth_password"    id="CiscoAuth_password"           size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var ciscoAuthPassword2  = $( '<input type="password" name="CiscoAuth_password_confirm" id="CiscoAuth_password_confirm"      size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var ciscoAuthEnablePwd1 = $( '<input type="password" name="CiscoAuth_enablepassword"   id="CiscoAuth_enablepassword"      size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var ciscoAuthEnablePwd2 = $( '<input type="password" name="CiscoAuth_enablepassword_confirm" id="CiscoAuth_enablepassword_confirm"  size="50" maxlength="250" />' ).get(0);

        accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser         , required: true  , element: ciscoAuthUserName   } );
        accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelPasswordField          , required: true  , element: ciscoAuthPassword1  } );
        accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField   , required: true  , element: ciscoAuthPassword2  } );
        accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelDiscoCredsEnablePassw1 , required: false , element: ciscoAuthEnablePwd1 } );
        accessForms.CiscoAuth.appendFieldRow( { label: i18n.labelDiscoCredsEnablePassw2 , required: false , element: ciscoAuthEnablePwd2 } );


        $(ciscoAuthPassword1).prop('required',true);
        $(ciscoAuthPassword2).prop('required',true);

        if (isEditMode) {
            $( ciscoAuthUserName  ).val( credential.properties.username );
        }

        /*
         * Fields for CiscoWorksAuth:
         */
        /** @type {Element} */ var ciscoWorksAuthUserName  = $( '<input type="text"     name="CiscoWorksAuth_username"         id="CiscoWorksAuth_username"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var ciscoWorksAuthPassword1 = $( '<input type="password" name="CiscoWorksAuth_password"         id="CiscoWorksAuth_password"         size="50" maxlength="250" />' ).get(0);
        /** @type {Element} */ var ciscoWorksAuthPassword2 = $( '<input type="password" name="CiscoWorksAuth_password_confirm" id="CiscoWorksAuth_password_confirm" size="50" maxlength="250" />' ).get(0);

        accessForms.CiscoWorksAuth.appendFieldRow( { label: i18n.labelDiscoCredsUser       , required: true , element: ciscoWorksAuthUserName  } );
        accessForms.CiscoWorksAuth.appendFieldRow( { label: i18n.labelPasswordField        , required: true , element: ciscoWorksAuthPassword1 } );
        accessForms.CiscoWorksAuth.appendFieldRow( { label: i18n.labelConfirmPasswordField , required: true , element: ciscoWorksAuthPassword2 } );

        $(ciscoWorksAuthPassword1).prop('required',true);
        $(ciscoWorksAuthPassword2).prop('required',true);

        if (isEditMode) {
            $( ciscoWorksAuthUserName  ).val( credential.properties.username );
        }

        // Event handler - type changed:
        var onTypeFieldChanged = function( event ) {
            /** @type {string}  */ var selected  = $( typeField ).val();
            /** @type {string}  */ var className = '';
            /** @type {boolean} */ var testable  = false;

            for ( /** @type {string} */ var name in accessForms ) {
                accessForms[ name ][ ( name === selected ) ? 'show' : 'hide' ]();
            }

            /*
             * Only some of the supported authentication types have test credentials support implemented.
             */
            switch( selected ) {
                case 'HostAuth'            : testable = true ; className = 'com.collation.platform.security.auth.HostAuth'                            ; break;
                case 'WindowsHostAuth'     : testable = true ; className = 'com.collation.platform.security.auth.WindowsHostAuth'                     ; break;
             // case 'JBossAuth'           : testable = false; className = 'com.collation.platform.security.auth.JBossAuth'                           ; break;
             // case 'OracleAppAuth'       : testable = false; className = 'com.collation.platform.security.auth.OracleAppAuth'                       ; break;
             // case 'WeblogicAuth'        : testable = false; className = 'com.collation.platform.security.auth.WeblogicAuth'                        ; break;
             // case 'WebSphereAuth1'      : testable = false; className = 'com.collation.platform.security.auth.WebSphereAuth'                       ; break;
             // case 'WebSphereAuth2'      : testable = false; className = 'com.collation.platform.security.auth.WebSphereAuth'                       ; break;
             // case 'WebSphereAuth3'      : testable = false; className = 'com.collation.platform.security.auth.WebSphereAuth'                       ; break;
             // case 'WasSshAuth'          : testable = false; className = 'com.collation.platform.security.auth.WasSshAuth'                          ; break;
             // case 'Db2Auth'             : testable = false; className = 'com.collation.platform.security.auth.Db2Auth'                             ; break;
             // case 'OracleAuth'          : testable = false; className = 'com.collation.platform.security.auth.OracleAuth'                          ; break;
             // case 'SybaseAuth'          : testable = false; className = 'com.collation.platform.security.auth.SybaseAuth'                          ; break;
             // case 'SqlServerAuth'       : testable = false; className = 'com.collation.platform.security.auth.SqlServerAuth'                       ; break;
             // case 'DominoServerAuth'    : testable = false; className = 'com.collation.platform.security.auth.DominoServerAuth'                    ; break;
             // case 'MQServerAuth'        : testable = false; className = 'com.collation.platform.security.auth.MQServerAuth'                        ; break;
             // case 'ExchangeServerAuth'  : testable = false; className = 'com.collation.platform.security.auth.ExchangeServerAuth'                  ; break;
                case 'SnmpAuth'            : testable = true ; className = 'com.collation.platform.security.auth.SnmpAuth'                            ; break;
                case 'SnmpV3Auth'          : testable = true ; className = 'com.collation.platform.security.auth.SnmpV3Auth'                          ; break;
             // case 'CCMSServerAuth'      : testable = false; className = 'com.collation.platform.security.auth.CCMSServerAuth'                      ; break;
                case 'CiscoAuth'           : testable = false; className = 'com.collation.platform.security.auth.CiscoAuth'                           ; break;
                case 'CiscoWorksAuth'      : testable = false; className = 'com.collation.platform.security.auth.CiscoWorksAuth'                      ; break;
             // case 'LdapAuth'            : testable = false; className = 'com.collation.platform.security.auth.LdapAuth'                            ; break;
             // case 'SLDServerAuth'       : testable = false; className = 'com.collation.platform.security.auth.SLDServerAuth'                       ; break;
             // case 'CSMServerAuth'       : testable = false; className = 'com.ibm.cdb.discover.auth.sys.hpc.cm.csmserver.CSMServerAuth'             ; break;
             // case 'VeritasClusterAuth'  : testable = false; className = 'com.collation.platform.security.auth.VeritasClusterAuth'                  ; break;
             // case 'ITMScopeAuth'        : testable = false; className = 'com.ibm.cdb.discover.auth.net.itmscope.ITMScopeAuth'                      ; break;
             // case 'SysImagerServerAuth' : testable = false; className = 'com.ibm.cdb.discover.auth.sys.hpc.cm.sysimagerserver.SysImagerServerAuth' ; break;
             // case 'VCAuth'              : testable = false; className = 'com.collation.platform.security.auth.VCAuth'                              ; break;
            }

            testForm[ testable ? 'show' : 'hide' ]();
            typeForm.putHiddenInput( 'ClassName', className );

            if (( selected === 'HostAuth' ) ||
                   ( selected === 'WindowsHostAuth' ) ) {
                onHostAuthAuthTypeChanged( null );
            }
        };

        $( typeField ).on( 'change', onTypeFieldChanged );

        /*
         * Event handler - scope set restriction type changed:
         */
        var onRestrictionTypeFieldChanged = $.proxy( function( event ) {
            /** @type {boolean} */ var isHostAuth          = !accessForms.HostAuth.isHidden();
            /** @type {boolean} */ var isPKI               = hostAuthTypePkiRadio.checked;
            /** @type {Element} */ var warnMsgElement      = ( isHostAuth ? hostAuthUploadKeyWarn : null );

            $( warnMsgElement ).empty();

            if (isHostAuth) {
                // For credential type HostAuth.......
                if (isPKI) {
                    // When PKI, get the PID of the current scope set restriction selection.
                    /** @type {string} */ let selected = $(scopeSetField).val();
                    /** @type {string} */ let selectedName = '';

                    /** @type {Array.<string>} */ let pkiCredentials = [];

                    $.each( this.cachedScopeSets, function( scopeSetIndex, thisScopeSet ) {
                        // Find the scope set that matches the current scope set restriction selection.  Then check
                        if (selected === thisScopeSet.pid) {
                            selectedName = thisScopeSet.name;

                            // Go through all credentials associated with this scope set to see if any are PKI.
                            $.each( thisScopeSet.credentials, function (credentialsIndex, thisCredential) {
                                if (TV.typeIsObject( typeof thisCredential                     ) && ( thisCredential            !== null ) &&
                                    TV.typeIsObject( typeof thisCredential.properties          ) && ( thisCredential.properties !== null ) &&
                                    TV.typeIsString( typeof thisCredential.properties.authtype ) && ( thisCredential.properties.authtype === 'authType_pki' ) ) {

                                    // For new credentials, always push the PKI credential.  For edit, only push credential if it
                                    // is not the credential being edited.
                                    if (isEditMode) {
                                        if (credential.pid !== thisCredential.pid) {
                                            pkiCredentials.push( thisCredential );
                                        }
                                    } else {
                                        pkiCredentials.push( thisCredential );
                                    }
                                }
                            } );
                        }

                    } );

                    if ((pkiCredentials !== null) && (pkiCredentials.length > 0)) {
                        // An existing credential with PKI was found, so put up warning message indicating the credential(s)
                        // that have PKI and the name of the current scope set restriction selection.
                        /** @type {string} */ let warnMsg          = '';
                        /** @type {string} */ let pkiCredsAsString = '';

                        $.each( pkiCredentials, function( credIndex, cred ) {
                            pkiCredsAsString += ( ( credIndex > 0 ) ? ', ' : '' ) + cred.name;
                        } );

                        warnMsg = tsa.bind( i18n.labelDiscoCredsPKICredExists, [ pkiCredsAsString, selectedName ] );

                        $( warnMsgElement ).append( IconFactory.buildSeverityIcon( { severity : 'WARN' } )
                                                  , document.createTextNode( '\xa0' + warnMsg ) );

                        $( warnMsgElement ).show();

                    } else {
                        $( warnMsgElement ).hide();
                    }
                }

                if (isPKI) {
                    // For PKI:
                    //     1) A scope set restriction is always required (set by onHostAuthAuthTypeChanged event handler)
                    //     2) The "Select Scope Set Restriction" pane should always be shown.
                    scopeSetForm.show();

                } else {
                    // For Password:
                    //     1) Show "Select Scope Set Restriction" pane when radio button for "Authentication type" is
                    //        "Password".
                    if (restrictionScopeRadio.checked) {
                        scopeSetForm.show();
                    } else {
                        scopeSetForm.hide();
                    }
                }

            } else {
                // For credential types other than HostAuth, "Select Scope Set Restriction" pane is always visible.
                // Just need to determine if "Restrict To Selected Scope Set" pane is visible based on setting in
                // "Select Scope Set Restriction" pane.
                if (restrictionScopeRadio.checked) {
                    scopeSetForm.show();
                } else {
                    scopeSetForm.hide();
                }

                // No PKI when not HostAuth, so hide PKI warning message.
                $( warnMsgElement ).hide();
            }
        }
        , this );

        $( restrictionAllRadio  ).add( restrictionScopeRadio ).on( 'change', onRestrictionTypeFieldChanged );
        $( scopeSetField ).on( 'change', onRestrictionTypeFieldChanged );

        /*
         * Event handler - HostAuth AuthType changed
         *
         * NOTE: When "Authentication Type" is PKI, it must be scoped to a specific scope set.  As a result,
         *       the "Select Scope Set Restriction" panel is only show when "Authentication Type" is Password.
         */
        var onHostAuthAuthTypeChanged = function( event ) {
            /** @type {boolean} */ var isHostAuth          = !accessForms.HostAuth.isHidden();
            /** @type {boolean} */ var isPKI               = hostAuthTypePkiRadio.checked;
            /** @type {boolean} */ var hideSSHKeyRow       = !isPKI;
            /** @type {boolean} */ var hideSSHKeyUpload    = hideSSHKeyRow;
            /** @type {boolean} */ var disableSSHKeyUpload = !isPKI;
            /** @type {boolean} */ var hideKeyTypeMessage  = isPKI && !isEditMode;
            /** @type {Element} */ var row                 = $( hostAuthUploadKey ).closest( 'tr' ).get(0);

            $( row                   )[ hideSSHKeyRow      ? 'hide' : 'show' ]();
            $( hostAuthUploadKey     )[ hideSSHKeyUpload   ? 'hide' : 'show' ]();
            $( hostAuthUploadKeyType )[ hideKeyTypeMessage ? 'hide' : 'show' ]();

            /*
             * Choose the right field label based on authentication type -- either "password" or "passphrase"
             */
            var labelValue = isPKI ? i18n.labelPassphraseField : i18n.labelPasswordField ;
            var labelNode  = $(hostAuthPassword1).closest('tr').find('label');
            labelNode.empty();
            if (isPKI) {
                labelNode.append(labelValue);
            } else {
                labelNode.append(labelValue,$("<span class='ibm-required'> *</span>").get(0));
            }

            labelValue = isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField ;
            labelNode  = $(hostAuthPassword2).closest('tr').find('label');
            labelNode.empty();
            if (isPKI) {
                labelNode.append(labelValue);
            } else {
                labelNode.append(labelValue, $("<span class='ibm-required'> *</span>").get(0));
            }

            /*
             * Set whether password / passphrase field is optional or required
             *    - passphrase is optional for PKI authentication type (scope set restricted).
             *    - password / passphrase is required for all other authentication types.
             */
            if (isPKI) {
                accessForms.HostAuth.removeValidation($(hostAuthPassword1).attr('id'));
                accessForms.HostAuth.removeValidation($(hostAuthPassword2).attr('id'));
            } else {
                accessForms.HostAuth.addValidationRule($(hostAuthPassword1).attr('id'), 'required');
                accessForms.HostAuth.addValidationRule($(hostAuthPassword2).attr('id'), 'required');
            }

            if (isPKI) {
                // Hide the "Select Scope Set Restriction" pane for PKI.  Force the "Restrict To Selected Scope Set"
                // to be shown.
                restrictionForm.hide();
                restrictionForm.putHiddenInput('RestrictionType', 'limit');

                scopeSetForm.show();

                // Force check to see if another PKI credential exists for the scope set restriction.
                onRestrictionTypeFieldChanged();

            } else {
                restrictionForm.show();
                restrictionForm.putHiddenInput('RestrictionType', null);
            }

            hostAuthUploadKey.disabled = disableSSHKeyUpload;
        };

        $( hostAuthTypePasswordRadio  ).add( hostAuthTypePkiRadio ).on( 'change', onHostAuthAuthTypeChanged );


        /*
         * Event handler Authentication protocol changed:
         */
        var onAuthProtoChanged = function( event ) {
            /** @type {string}  */ var selected   = $( snmpV3AuthAuthProto ).val();

            if (selected === 'MD5' || selected === 'SHA') {
                var labelNode = $(snmpV3AuthPassword1).closest('tr').find('label');

                accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPassword1).attr('id'), "required" );
                labelNode.empty();
                labelNode.append(i18n.labelPasswordField, $("<span class='ibm-required'> *</span>").get(0));

                labelNode = $(snmpV3AuthPassword2).closest('tr').find('label');
                accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPassword2).attr('id'), "required" );
                labelNode.empty();
                labelNode.append(i18n.labelConfirmPasswordField, $("<span class='ibm-required'> *</span>").get(0));

                labelNode = $(snmpV3AuthPrivPwd1).closest('tr').find('label');
                accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPrivPwd1).attr('id'), "required" );
                labelNode.empty();
                labelNode.append(i18n.labelDiscoCredsPrivatePassw1, $("<span class='ibm-required'> *</span>").get(0));

                labelNode = $(snmpV3AuthPrivPwd2).closest('tr').find('label');
                accessForms.SnmpV3Auth.addValidationRule( $(snmpV3AuthPrivPwd2).attr('id'), "required" );
                labelNode.empty();
                labelNode.append(i18n.labelDiscoCredsPrivatePassw2, $("<span class='ibm-required'> *</span>").get(0));

            } else if (selected === 'None') {
                var labelNode = $(snmpV3AuthPassword1).closest('tr').find('label');

                accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPassword1).attr('id') );
                labelNode.empty();
                labelNode.append(i18n.labelPasswordField);

                labelNode = $(snmpV3AuthPassword2).closest('tr').find('label');
                accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPassword2).attr('id') );
                labelNode.empty();
                labelNode.append(i18n.labelConfirmPasswordField);

                labelNode = $(snmpV3AuthPrivPwd1).closest('tr').find('label');
                accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPrivPwd1).attr('id') );
                labelNode.empty();
                labelNode.append(i18n.labelDiscoCredsPrivatePassw1);

                labelNode = $(snmpV3AuthPrivPwd2).closest('tr').find('label');
                accessForms.SnmpV3Auth.removeValidation( $(snmpV3AuthPrivPwd2).attr('id') );
                labelNode.empty();
                labelNode.append(i18n.labelDiscoCredsPrivatePassw2);
            }
        };

        $( snmpV3AuthAuthProto ).on( 'change', onAuthProtoChanged );

        /*
         * Event handler - Test Button:
         */
        $( testButton ).on( 'click', function( event ) {
            /** @type {string} */ var action = firstForm.getActionAttribute();

            firstForm.putHiddenInput( 'testCredential', 'true' );
            firstForm.setActionAttribute( 'credentials/ajaxTestCredential' );
            firstForm.submit();
            firstForm.setActionAttribute( action );
        } );

        // Loading the Scope Sets in the Scope Set Selector...
        $( getScopeSetsCmd ).on( 'executionSucceeded', $.proxy( function( event )  {
            if (TV.typeIsObject( typeof event.response.payload.scopeSets ))  {
                /** @type {Array.<Object>} */ var currentScopeSets = event.response.payload.scopeSets;

                if (TV.typeIsObject( typeof currentScopeSets ) && TV.isInstanceOf( currentScopeSets, Array )) {
                    logger.info( 'Populating scope set selector (' + currentScopeSets.length + ' scope set(s))...' );
                    $( scopeSetField ).empty();
                    this.cachedScopeSets = {};

                    $.each( currentScopeSets, $.proxy( function( scopeSetIndex, scopeSet ) {
                        if (TV.typeIsString( typeof scopeSet.displayName ) &&
                            TV.typeIsString( typeof scopeSet.guid )) {
                            /** @type {Element} */ var option = document.createElement( 'option' );

                            $( option ).text( scopeSet.displayName );
                            $( option ).prop( 'value', scopeSet.guid );
                            $( scopeSetField ).append( option );

                            this.cachedScopeSets[ scopeSet.guid ] = scopeSet;

                        } else {
                            logger.warn( 'Skipping invalid scope set (' + 'index'       + ': ' + scopeSetIndex        + '; '
                                                                        + 'guid'        + ': ' + scopeSet.guid        + '; '
                                                                        + 'displayName' + ': ' + scopeSet.displayName + ').' );
                        }
                    }
                    , this ) );

                    if (isEditMode && TV.typeIsObject( typeof credential.scopeSet ) && ( credential.scopeSet !== null )) {
                        $( scopeSetField ).val( credential.scopeSet.pid );
                    }
                }
            }

            // Triggering the event handlers for the first time, for initialization purposes:
            onTypeFieldChanged( null );
            onRestrictionTypeFieldChanged( null );
            onHostAuthAuthTypeChanged( null );

        }
        , this ) );

        // Retrieving Scope Sets from server...
        getScopeSetsCmd.execute();

        // Return the form...
        return firstForm;
    };

    //-------------------------------------------------------------------------

    /**
     * Convert authentication-type value to i18n label.
     * @param {string} authtype Authentication type value.
     * @return {string}         The i18n label, or the authentication type value
     *                          unchanged if an i18n label was not found for it.
     */
    tsa.cmd.handlers.CredentialsHandler.i18nAuthType = function ( credential ) {
        /** @type {string} */  var i18nLabel = '';
        /** @type {string} */  var authType  = '';
        if (TV.typeIsString (typeof credential)) {
            authType = credential;

        } else if (!TV.typeIsUndefined( typeof credential.properties.authtype ))  {
            /** @type {boolean} */ var isPKI             = ( credential.properties.authtype.indexOf( 'authType_pki' ) === 0 );
            /** @type {boolean} */ var restricted        = ( TV.typeIsString( typeof credential.scopeSetName ) &&
                                                           ( credential.scopeSetName.length > 0 ) );
            /** @type {Object}  */ var scopeSet          = ( ( restricted && TV.typeIsObject( typeof credential.scopeSet ) ) ? credential.scopeSet : null );
            /** @type {string}  */ var authType          = credential.properties.authtype;
            /** @type {boolean} */ var scopeSetHasSSHKey = ( ( scopeSet !== null ) && ( scopeSet.hasSSHKey === true ) );

        } else if (credential.authClassSimpleName === 'WindowsHostAuth') {
            // Windows only supports password authentication
            authType = "authType_password";

        } else {
            authType = credential.authClassLabel;
        }

        switch( authType ) {
            case 'authType_password' : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePassword  ; break;
            case 'authType_pki'      : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePkiScoped ; break;
            default                  : i18nLabel = authType;
        }

        return i18nLabel;
    };

    //-------------------------------------------------------------------------

    /**
     * Convert credential-type value to i18n label.
     * @param {string} credtype Credential type value.
     * @return {string}         The i18n label, or the credential type value
     *                          unchanged if an i18n label was not found for it.
     */
    tsa.cmd.handlers.CredentialsHandler.i18nCredType = function ( credential ) {
        /** @type {string} */  var i18nLabel = '';
        /** @type {string} */  var credType  = '';
        if (TV.typeIsString (typeof credential)) {
            credType = credential;

        } else if (!TV.typeIsUndefined( typeof credential.authClassSimpleName ))  {
           credType = credential.authClassSimpleName;
        }

        switch( credType ) {
            case 'HostAuth'        : i18nLabel = tsa.i18n.labelDiscoCredsTypeHostAuth  ; break;
            case 'WindowsHostAuth' : i18nLabel = tsa.i18n.labelDiscoCredsTypeWindowsHostAuth ; break;
            case 'SnmpAuth'        : i18nLabel = tsa.i18n.labelDiscoCredsTypeSnmpAuth ; break;
            case 'SnmpV3Auth'      : i18nLabel = tsa.i18n.labelDiscoCredsTypeSnmpV3Auth ; break;
            case 'CiscoAuth'       : i18nLabel = tsa.i18n.labelDiscoCredsTypeCiscoAuth ; break;
            case 'CiscoWorksAuth'  : i18nLabel = tsa.i18n.labelDiscoCredsTypeCiscoWorksAuth ; break;
            default                : i18nLabel = credType;
        }

        return i18nLabel;
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.CredentialsHandler;
  }
);
