/*
 * IBM Technical Support Appliance Version 2 Release 8 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2021
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery' ,
    'tsa/TypeVerificator' ,
    'tsa/cmd/handlers/AbstractHandler' ,
    'tsa/cmd/Command' ,
    'tsa/cmd/ShowCommand' ,
    'tsa/widgets/ButtonFactory' ,
    'tsa/widgets/ContentArea' ,
    'tsa/widgets/FormPanel' ,
    'tsa/widgets/IconFactory' ,
    'tsa/widgets/TablePanel'
  ],

  function( $,
            TV,
            AbstractHandler,
            Command,
            ShowCommand,
            ButtonFactory,
            ContentArea,
            FormPanel,
            IconFactory,
            TablePanel )
{
    //-------------------------------------------------------------------------

    var logger = tsa.getLogger('CredentialshmcHandler');

    //-------------------------------------------------------------------------

    /**
     *  @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *
     *  @param {Object} parameters The parameters.
     */
    tsa.cmd.handlers.CredentialshmcHandler = function(parameters) {
        if ((!TV.typeIsObject(typeof parameters)) || (parameters === null)) {
            parameters = /** @type {?} */ ( {} );
        }

        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call(this, parameters);
    };

    //-------------------------------------------------------------------------

    // Inheriting:
    tsa.cmd.handlers.CredentialshmcHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.CredentialshmcHandler.prototype ).constructor = tsa.cmd.handlers.CredentialshmcHandler;

    //-------------------------------------------------------------------------


    /**
     * Show main screen (list the credentials).  NOT USED.  Credentials viewed via the HMC Dynamic Scope edit
     * function (ScopsethmcHandler.js).
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype._index = function() {

    };



    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** ADD CREDENTIAL SCREEN                                                  ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Add new credential screen.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype._new = function() {
        /** @type {Object} */ var i18n = tsa.i18n;

        /** @type {string} */ var pid            = this.getParameter('pid');
        /** @type {string} */ var credentialType = this.getParameter('type');
        /** @type {string} */ var scopeSetName   = this.getParameter('scopeSetName');

        /** @type {tsa.widgets.ContentArea} */
        var contentArea = new ContentArea( { title   : i18n.labelDiscoHMCCredsNewCredsHeading,
                                             helpDoc : 'sshelp_discovery_hmc_credentials_new_panel.html' } );

        /** @type {tsa.widgets.FormPanel} */
        var credForm = this.createCredentialForm(credentialType, scopeSetName, null, pid);

        /** @type {tsa.widgets.ButtonFactory} */
        var buttonContainer = ButtonFactory.buildButtonContainer();

        credForm.setActionAttribute( 'credentialshmc/postCredential' );

        /*
         * Create "save" and "cancel" buttons.
         */
        /** @type {tsa.widgets.ButtonFactory} */
        var saveButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      label   : i18n.buttonSave } );

        $(saveButton).on('click', function () {
            // Indicate normal save request (not a test credential).
            credForm.putHiddenInput( 'testCredential', null );

            var inputValue = $("input[name='name']").val();
            if (inputValue.length === 0) {
                credForm.validate();
            } else {
                $("input[name='name']").val(inputValue);
                credForm.submit();
            }
        } );

        /** @type {tsa.widgets.ButtonFactory} */
        var cancelButton = ButtonFactory.buildButton( { type    : 'cancel',
                                                        label   : i18n.buttonCancel } );

        $(cancelButton).on('click', function () {
            ( new ShowCommand( { action              : 'scopesetshmc/show',
                                 clearSystemMessages : true,
                                 parameters          : { pid : pid } } ) ).execute();
        } );

        $(buttonContainer).append(saveButton);
        $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
        $(buttonContainer).append(cancelButton);

        // Appending content...
        contentArea.append( credForm.getDomNode() );
        contentArea.append( buttonContainer );

        /*
         * Add successful:
         *     - When save button was clicked, print success message, then show the credentials list.
         *     - When test credential button was clicked, stay at current panel so user can later save the credential if desired.
         */
        $(credForm).on('executionSucceeded', function() {
            if (credForm.getHiddenInput("testCredential") === null) {
                var credentialName = $("input[name='name']").val();
                tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoHMCCredsCreated, credentialName ) } );

                ( new ShowCommand( { action              : 'scopesetshmc/show',
                                     clearSystemMessages : false,
                                     parameters          : { pid : pid } } ) ).execute();
            }
        } );

        /*
         * Add / Test failed.  Show which fields are in error.
         */
        $(credForm).on('executionFailed', function(event) {
            if (TV.typeIsObject(typeof event.response.payload.errors)) {
                for (var fieldName in event.response.payload.errors) {
                    credForm.showFieldError(fieldName, event.response.payload.errors[fieldName]);
                }
                delete event.response.payload.errors;
            }
        } );

        this.show( contentArea.getDomNode() );
    };



    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** SHOW CREDENTIAL SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Show details of an existing credential.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype._show = function() {
        /** @type {Object} */ var i18n = tsa.i18n;

        /** @type {string} */ var pid   = this.getParameter( 'entityId' );
        /** @type {string} */ var sepid = this.getParameter( 'pid' );

        /** @type {tsa.widgets.ContentArea} */
        var contentArea  = new ContentArea( { title   : i18n.labelDiscoHMCCredsHeading,
                                              helpDoc : 'sshelp_discovery_hmc_credentials_show_panel.html' } );
        /** @type {tsa.widgets.TablePanel}  */
        var generalTable = new TablePanel( { title   : i18n.labelGeneralPaneHeading,
                                             columns : [ { id: 'name'  , label: '' , width: '30%' , sortable: false },
                                                         { id: 'value' , label: '' , width: '70%' , sortable: false } ] } );
        /** @type {Element} */ var buttonContainer = ButtonFactory.buildButtonContainer();

        /*
         * Get specific credential.
         */
        var getCredentialCmd = new Command( { action              : 'credentials/getCredential',
                                              parameters          : { 'pid' : pid },
                                              clearSystemMessages : false } );

        $( getCredentialCmd ).on( 'executionSucceeded', $.proxy( function( event ) {
            var credential = event.response.payload.credential;

            if ( TV.typeIsObject( typeof credential ) && ( credential !== null ) )  {
                var scopeSetSpan = document.createElement( 'span' );
                var userName     = credential.userName;
                var scopeSetName = credential.scopeSetName;

                if ( userName     === null ) userName     = '';
                if ( scopeSetName === null ) scopeSetName = '';

                // Trim off hidden portion of scope set name
                var displayScopeSetName = this.getDisplayableScopeSetName(scopeSetName);

                $( scopeSetSpan ).text( displayScopeSetName );

                var credentialTarget = this.i18nCredentialType(credential.name);
                var credentialName   = credential.name.substring(0, credential.name.indexOf("_SCOPESET_"));

                /*
                 * Populate the rows of the show panel.
                 */
                generalTable.appendRow( [ i18n.labelNameField               , credentialName ] );
                generalTable.appendRow( [ i18n.labelTypeField               , credentialTarget ] );
                generalTable.appendRow( [ i18n.labelUserNameFieldColon      , userName ] );
                generalTable.appendRow( [ i18n.labelScopeSetField           , scopeSetSpan ] );
                generalTable.appendRow( [ i18n.labelAuthenticationTypeField , this.i18nAuthType(credential.properties.authtype) ] );

                // If TSA web UI is in debug mode, show a few more fields.
                if ( tsa.isDebugModeOn() ) {
                    generalTable.appendRow( [ i18n.labelPidDebugField   , credential.pid           ] );
                    generalTable.appendRow( [ i18n.labelClassDebugField , credential.authClassName ] );
                }
            }

            var backButton = ButtonFactory.buildButton( { type    : 'back',
                                                          grayout : true,
                                                          label   : i18n.buttonGoBack } );

            $(backButton).on('click', function () {
                ( new ShowCommand( {
                    action     : 'scopesetshmc/show',
                    parameters : { pid : sepid   }
                } ) ).execute();
            } );

            var editButton = ButtonFactory.buildButton( { type  : 'arrow',
                                                          label : i18n.buttonDiscoCredsEdit } );

            $(editButton).on('click', function () {
                tsa.cmd.handlers.CredentialshmcHandler.prototype.editCredentialClicked( credential.pid, sepid );
            } );

            $( buttonContainer ).append( backButton );
            $( buttonContainer ).append( ButtonFactory.buildButtonSeparator() );
            $( buttonContainer ).append( editButton );

            contentArea.append( generalTable.getDomNode() );
            contentArea.append( buttonContainer );
        }, this )

        );

        getCredentialCmd.execute();

        this.show( contentArea.getDomNode() );
    };


    //-------------------------------------------------------------------------

    /********************************************************************************/
    /********************************************************************************/
    /****                                                                        ****/
    /**** EDIT CREDENTIAL SCREEN                                                 ****/
    /****                                                                        ****/
    /********************************************************************************/
    /********************************************************************************/

    /**
     * Edit an existing credential.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype._edit = function(){
        var i18n = tsa.i18n;

        var contentArea  = new ContentArea( { title   : i18n.labelDiscoHMCEditCredsHeading,
                                              helpDoc : 'sshelp_discovery_hmc_credentials_edit_panel.html' } );
        var pid          = this.getParameter('pid');
        var credentialId = this.getParameter('entityId');

        var credentialForm  = null;
        var buttonContainer = ButtonFactory.buildButtonContainer();

        var getCredentialCommand = new Command( { action              : 'credentials/getCredential',
                                                  parameters          : { 'pid' : credentialId },
                                                  clearSystemMessages : false } );

        $( getCredentialCommand ).on( 'executionSucceeded', $.proxy(function(event) {
            var credentialType = this.i18nCredentialType(event.response.payload.credential.name);
            var scopeSetName   = this.getDisplayableScopeSetName(event.response.payload.credential.scopeSetName);

            credentialForm = this.createCredentialForm( credentialType, scopeSetName, event.response.payload.credential, pid );

            credentialForm.setActionAttribute( 'credentialshmc/putCredential' );

            // If edit succeeds, print success message, then show the credentials list.
            $( credentialForm ).on( 'executionSucceeded', function( ) {
                if (credentialForm.getHiddenInput("testCredential") === null) {
                    var credentialName = $( credentialForm.getDomNode() ).find( '[name=\'name\']' ).val();
                    var trimmedCredentialName = credentialName.substring(0, credentialName.indexOf("_SCOPESET_"));
                    tsa.printSysMessage( { level: 'info', message: tsa.bind( tsa.i18n.labelDiscoHMCCredsUpdated, trimmedCredentialName ) } );

                    ( new ShowCommand( { action              : 'scopesetshmc/show',
                                         parameters          : { pid : pid },
                                         clearSystemMessages : false } ) ).execute();
                }
            } );

            // Appending content...
            contentArea.append( credentialForm.getDomNode() );
            contentArea.append( buttonContainer );
        } , this ) );

        /*
         * Create "save" and "cancel" buttons.
         */
        var saveEditButton = ButtonFactory.buildButton( { type  : 'arrow',
                                                          label : i18n.buttonSave } );

        $(saveEditButton).on('click', function () {
            credentialForm.putHiddenInput( 'testCredential', null );
            credentialForm.submit();
        } );

        var cancelEditButton = ButtonFactory.buildButton( { type  : 'cancel',
                                                            label : i18n.buttonCancel} );

        $(cancelEditButton).on('click', function () {
            ( new ShowCommand( { action : 'scopesetshmc/show',
                                 parameters : { pid : pid}
                               } ) ).execute();
        } );

        $(buttonContainer).append(saveEditButton);
        $(buttonContainer).append(ButtonFactory.buildButtonSeparator());
        $(buttonContainer).append(cancelEditButton);

        // Get the credential...
        getCredentialCommand.execute();

        this.show( contentArea.getDomNode() );
    };


    /**
     * Edit credential - event handler.
     * @param {string} credentialPID The credential PID.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype.editCredentialClicked = function( credentialPID , pid) {
        ( new ShowCommand( { action : 'credentialshmc/edit/' + credentialPID + '/',
                             parameters : { pid : pid } } ) ).execute();
    };


    //-------------------------------------------------------------------------

    /*****************************************************************************/
    /* CREATE CREDENTIAL FORM:                                                   */
    /*                                                                           */
    /* Used for both adding a new credential or editing an existing credential.  */
    /*                                                                           */
    /* The credential panel consists of 3 panes:                                 */
    /*    - General                                                              */
    /*    - Enter Access Information                                             */
    /*    - Test Access Credentials                                              */
    /*                                                                           */
    /* Both add and edit panels have the same fields:                            */
    /*    - for add,  "Name" field is input                                      */
    /*    - for edit, "Name" field has human readable version of credential      */
    /*      name.  The value of "Name" cannot be changed.                        */
    /*                                                                           */
    /* @param type Indicates whether credential is HMC, AIX, Linux, or VIOS.     */
    /* @param scopeSetName - Display name of the HMC Dynamic Scope Set.          */
    /* @param credential -                                                       */
    /* @param pid -                                                              */
    /*****************************************************************************/

    tsa.cmd.handlers.CredentialshmcHandler.prototype.createCredentialForm = function( type, scopeSetName, credential, pid ) {
        var i18n = tsa.i18n;

        var isEditMode     = TV.typeIsObject( typeof credential ) && ( credential !== null );
        var generalForm    = null;
        var nameForm       = new FormPanel( { title       : i18n.labelNamePaneHeading,
                                              description : i18n.labelDiscoHMCCredsNameDesc } );
        var testForm       = new FormPanel( { title       : i18n.labelDiscoCredsTestHeading,
                                              description : i18n.labelDiscoCredsTestDesc } );
        var accessInfoForm = new FormPanel( { title: i18n.labelDiscoCredsEnterAccessHeading,
                                              description: i18n.labelDiscoCredsHostAuthEnterDesc} );


        var matchingScopeSet = null;
        var credentialOsName = null;


        accessInfoForm.putHiddenInput("dynamicScopeSetName", scopeSetName);

        /*
         * Determine OS name for add credentials.  Not needed for edit credential since the hidden
         * credential name includes the OS name in it.
         */
        if (type == tsa.i18n.labelDiscoHMCCredsHmcType) {
            credentialOsName = "_HMC";
        } else if (type == tsa.i18n.labelDiscoHMCCredsAixType) {
            credentialOsName = "_AIX";
        } else if (type == tsa.i18n.labelDiscoHMCCredsLinuxType) {
            credentialOsName = "_LINUX";
        } else if (type == tsa.i18n.labelDiscoHMCCredsViosType) {
            credentialOsName = "_VIOS";
        }

        if (!isEditMode) {
            accessInfoForm.putHiddenInput("credentialOsName", credentialOsName);
        }

        var firstForm = nameForm;

        nameForm.linkTo( accessInfoForm );
        accessInfoForm.linkTo( testForm );

        if (isEditMode) {
            nameForm.putHiddenInput("scopename", credential.scopeSet.pid);

            generalForm = new FormPanel( { title: i18n.labelDiscoCredsGeneralHeading } );
            generalForm.linkTo( nameForm );
            firstForm = generalForm;
        }

        // Fields (common):
        var nameField = $( '<input type="text" id="name" name="name" size="50" maxlength="250" />' ).get(0);
        var typeSpan  = document.createElement( 'span' );

        var testhostField = $( '<input type="text" id="ipAddress" name="ipAddress" size="50" maxlength="250" />' ).get(0);

        /*
         * Set-up test credential button.
         */
        var testButton = this.createTestButton(firstForm, 'credentialshmc/ajaxTestCredential');
        var testButtonCon = ButtonFactory.buildButtonContainer();

        $(testButtonCon).append(testButton);

        $(typeSpan).text(type);

        nameForm.appendFieldRow( { label: i18n.labelDiscoCredsName, required: true, element: nameField } );
        nameForm.appendFieldRow( { label: i18n.labelDiscoCredsType,                 element: typeSpan } );

        testForm.appendFieldRow( { label: i18n.labelDiscoCredsTestIP, required: false , element: testhostField } );
        testForm.appendHR();
        testForm.appendJointRow(testButtonCon);

        var dynamicScopeSetNameSpan = document.createElement('span');
        $(dynamicScopeSetNameSpan).css( { 'font-weight' : 'bold' } );

        if (isEditMode) {
            var nameSpan = document.createElement('span');

            $(nameSpan).css( { 'font-weight' : 'bold' } );

            $(nameSpan).text(credential.name.substring(0, credential.name.indexOf("_SCOPESET_")));

            $(dynamicScopeSetNameSpan).text(scopeSetName);

            generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralName, element: nameSpan } );
            generalForm.appendFieldRow( { label: i18n.labelDiscoCredsGeneralType, element: typeSpan } );
            generalForm.appendFieldRow( { label: i18n.labelScopeSetField,         element: dynamicScopeSetNameSpan } );

            firstForm.putHiddenInput( 'pid' , credential.pid );
            firstForm.putHiddenInput( 'name', credential.name );

            $(nameField).val(credential.name);

            nameForm.hide();

        } else {
            $(dynamicScopeSetNameSpan).text(scopeSetName);
            nameForm.appendFieldRow( { label: i18n.labelScopeSetField, element: dynamicScopeSetNameSpan } );
        }

        /*
         * Fields for HostAuth:
         */

        // Radio buttons for selecting authentication type.
        /** @type {Element} */ var authTypePasswordRadio = document.createElement( 'input' );
        /** @type {Element} */ var authTypePkiRadio      = document.createElement( 'input' );

        var hostAuthUserName  = $('<input type="text"     id="HostAuth_username"         name="HostAuth_username"         size="50" maxlength="250" />' ).get(0);
        var hostAuthPassword1 = $('<input type="password" id="HostAuth_password"         name="HostAuth_password"         size="50" maxlength="250" />' ).get(0);
        var hostAuthPassword2 = $('<input type="password" id="HostAuth_password_confirm" name="HostAuth_password_confirm" size="50" maxlength="250" />' ).get(0);

        var hostAuthUploadKey     = $('<input type="file" id="HostAuth_sshKey" name="HostAuth_sshKey" />').get(0);
        var hostAuthUploadKeyType = document.createElement('p');
        var hostAuthUploadKeyWarn = document.createElement('p');
        var hostAuthUploadKeyDiv  = document.createElement('div');

        // Initially hide PKI exists warning since on add credential the authentication type is initially set to "Password".
        $(hostAuthUploadKeyWarn).empty();
        $(hostAuthUploadKeyWarn)[ 'hide' ]();

        $(hostAuthUploadKeyDiv ).css( { 'font-size'        : '1.33em' } );
        $(hostAuthUploadKeyWarn).css( { 'margin-top'       : '0.80em',
                                        'border'           : '1px solid #c0c000',
                                        'border-radius'    : '5px',
                                        'background-color' : '#ffffc0' } );

        $(hostAuthUploadKeyDiv).append(hostAuthUploadKeyType);
        $(hostAuthUploadKeyDiv).append(hostAuthUploadKey    );
        $(hostAuthUploadKeyDiv).append(hostAuthUploadKeyWarn);


        // Authentication type radio buttons...
        authTypePasswordRadio.type  = 'radio';
        authTypePasswordRadio.name  = 'HostAuth_authtype';
        authTypePasswordRadio.value = 'authType_password';

        authTypePkiRadio.type  = 'radio';
        authTypePkiRadio.name  = 'HostAuth_authtype';
        authTypePkiRadio.value = 'authType_pki';

        if (isEditMode) {
            // Determine the existing credential authentication type and select the right radio button.
            var authType = credential.properties.authtype;
            if (authType === "authType_pki") {
                authTypePkiRadio.checked = true;
            } else {
                authTypePasswordRadio.checked = true;
            }
        } else {
            // On create credential, password is always the default.
            authTypePasswordRadio.checked = true;
        }

        accessInfoForm.appendFieldRow( {
            label    : i18n.labelAuthenticationTypeField
          , required : true
          , aRequire : false
          , element  : ( function()
                         {
                           /** @type {Element} */ var span          = document.createElement( 'span' );
                           /** @type {Element} */ var passwordLabel = document.createElement( 'label' );
                           /** @type {Element} */ var pkiLabel      = document.createElement( 'label' );

                           $( passwordLabel ).append( authTypePasswordRadio, document.createTextNode( i18n.labelCredentialsAuthenticationTypePassword  ) );
                           $( pkiLabel      ).append( authTypePkiRadio,      document.createTextNode( i18n.labelCredentialsAuthenticationTypePkiScoped ) );
                           $( span          ).append( passwordLabel, document.createElement( 'br' ), pkiLabel );
                           $( span          ).addClass( 'ibm-input-group' );
                           $( span          ).attr( 'role', 'radiogroup' );
                           $( span          ).attr( 'aria-label', i18n.scheduleDayMode );

                           return span;
                         } )()
        } );

        accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsUser                , required: true  , element: hostAuthUserName     } );
        accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw1              , required: true  , element: hostAuthPassword1    } );
        accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsPassw2              , required: true  , element: hostAuthPassword2    } );
        accessInfoForm.appendFieldRow( { label: i18n.labelDiscoCredsSSHKey              , required: true  , element: hostAuthUploadKeyDiv } );

        // Common value initialization for fields of HostAuth:
        if (isEditMode) {
            var usernameField = hostAuthUserName ;
            var scopeSet      = ((TV.typeIsObject(typeof credential.scopeSet)) ? credential.scopeSet : null);
            var authType      = credential.properties.authtype;

            $(usernameField).val(credential.properties.username);
         }

        /*
         * Event handler - credential Authorization Type has changed.
         *     - Display the right fields for the authorization type.
         *     - Check if the new or edited credential will cause a conflicting PKI credential.  TSA only allows one
         *       PKI credential per scope set, so the conflicting PKI must be identified.
         */
        var HostAuth_authtypeChanged = function() {

            /** @type {boolean} */ var isPKI = false;
            if (authTypePkiRadio.checked) {
                isPKI = true;
            }

            //
            // All checks are really based on whether this is a password or PKI credential, but for code clarity
            // purposes, create some variables that indicate what portion of the credential page should be shown
            // or hidden.
            //
            /** @type {boolean} */ var hideSSHKeyRow       = !isPKI;
            /** @type {boolean} */ var hideSSHKeyUpload    = hideSSHKeyRow;
            /** @type {boolean} */ var disableSSHKeyUpload = !isPKI;
            /** @type {boolean} */ var hideKeyTypeMessage  = isPKI;

            // panel elements
            var row                 = $(hostAuthUploadKey).closest('tr').get(0);

            $(row)                  [ hideSSHKeyRow      ? 'hide' : 'show' ]();
            $(hostAuthUploadKey)    [ hideSSHKeyUpload   ? 'hide' : 'show' ]();
            $(hostAuthUploadKeyType)[ hideKeyTypeMessage ? 'hide' : 'show' ]();

            /*
             * Choose the right field label based on authentication type -- either "password" or "passphrase"
             */
            var labelValue = isPKI ? i18n.labelPassphraseField : i18n.labelPasswordField ;
            var labelNode  = $(hostAuthPassword1).closest( 'tr' ).find( 'label' );
            labelNode.empty();
            if (isPKI) {
                // Passphrase is optional for PKI
                labelNode.append(labelValue);
            } else {
                labelNode.append(labelValue,$("<span class='ibm-required'> *</span>").get(0));
            }

            labelValue = isPKI ? i18n.labelConfirmPassphraseField : i18n.labelConfirmPasswordField ;
            labelNode  = $(hostAuthPassword2).closest( 'tr' ).find( 'label' );
            labelNode.empty();
            if (isPKI) {
                // Passphrase is optional for PKI
                labelNode.append(labelValue);
            } else {
                labelNode.append(labelValue,$("<span class='ibm-required'> *</span>").get(0));
            }

            /*
             * Set whether password / passphrase field is optional or required
             *    - passphrase is optional for PKI authentication type (scope set restricted).
             *    - password / passphrase is required for all other authentication types.
             */
            if (isPKI) {
                accessInfoForm.removeValidation( $(hostAuthPassword1).attr('id') );
                accessInfoForm.removeValidation( $(hostAuthPassword2).attr('id') );
            } else {
                accessInfoForm.addValidationRule( $(hostAuthPassword1).attr('id'), 'required' );
                accessInfoForm.addValidationRule( $(hostAuthPassword2).attr('id'), 'required' );
            }

            /*
             * Function to get list of PKI credentials for the scope set.
             */
            var retrievePKICredentialsFor = function( scopeSetData ) {
                // Array of credentials with PKI (should only be a max of 1).
                var pkiCredentials = [];

                if ( TV.typeIsObject(typeof scopeSetData) && (scopeSetData !== null) &&
                        TV.typeIsObject( typeof scopeSetData.credentials ) && TV.isInstanceOf( scopeSetData.credentials, Array ) ) {
                    $.each( scopeSetData.credentials, function( credIndex, cred ) {
                        if (    TV.typeIsObject(typeof cred                    ) && (cred            !== null) &&
                                TV.typeIsObject(typeof cred.properties         ) && (cred.properties !== null) &&
                                TV.typeIsString(typeof cred.properties.authtype) &&
                                (cred.properties.authtype === 'authType_pki') ) {
                            /*
                             * PKI credential.  Skip current credential when in edit mode.
                             */
                            if (isEditMode && credential.pid == cred.pid) {
                                logger.info("skipping pid: " + cred.pid);
                            } else {
                                pkiCredentials.push(cred);
                            }
                        }
                    } );
                }

                return pkiCredentials;
            };

            /*
             * Determine if a warning message for conflicting PKI credentials is required.  If a conflict is detected, adding
             * the new or edited credential will cause the conflicting credential to be deleted.
             */
            var warnMsgElement = hostAuthUploadKeyWarn ;

            // Array of PKI credentials for credential being created or edited.
            var scopeSetPkiCreds    = ( ( isPKI && ( matchingScopeSet !== null ) ) ? retrievePKICredentialsFor( matchingScopeSet ) : [] );

            var scopeSetHasPkiCreds = ( ( scopeSetPkiCreds !== null ) && ( scopeSetPkiCreds.length > 0 ) );

            var pkiCredsAsString    = '';

            $( warnMsgElement ).empty();
            $( warnMsgElement )[ scopeSetHasPkiCreds ? 'show' : 'hide' ]();

            if (scopeSetHasPkiCreds) {
                var warnMsg = '';

                $.each( scopeSetPkiCreds, function( credIndex, cred ) {
                    var displayCredName = cred.name.substring(0, cred.name.indexOf("_SCOPESET_"));
                    pkiCredsAsString += ( ( credIndex > 0 ) ? ', ' : '' ) + displayCredName;
                } );

                var displayMatchingScopeSetName = matchingScopeSet.name.replace(credentialOsName, "");

                warnMsg = tsa.bind( i18n.labelDiscoCredsPKICredExists, [ pkiCredsAsString, displayMatchingScopeSetName ] );

                $( warnMsgElement ).append( IconFactory.buildSeverityIcon( { severity : 'WARN' } ), document.createTextNode( '\xa0' + warnMsg ) );
            }

            hostAuthUploadKey.disabled = disableSSHKeyUpload;
        };

        $( authTypePasswordRadio  ).add( authTypePkiRadio ).on( 'change', HostAuth_authtypeChanged );

        $(authTypePasswordRadio).trigger("change");

        /*
         * Use the get Scope Sets action to find the scope set for this dynamic scope set.
         */
        var getScopeSetsCmd = new Command( { action     : 'scopesets/getScopeSets',
                                             parameters : { 'withCredentials' : 'true',
                                                            'filter'          : 'HMC' },
                                             clearSystemMessages : false } );

        $( getScopeSetsCmd ).on( 'executionSucceeded', $.proxy( function( event ) {

            logger.info("execute succeeded for get scope sets command");

            var hmcScopeSet = null;

            if ( TV.typeIsObject( typeof event.response.payload.scopeSets ) ) {
                // Array of HMC Dynamic Scope Set related scope sets.
                var scopeSets = event.response.payload.scopeSets;

                if ( TV.typeIsObject( typeof scopeSets ) && TV.isInstanceOf( scopeSets, Array ) ) {
                    logger.info( 'Populating scope set selector (' + scopeSets.length + ' scope set(s))...' );

                    /*
                     * Find the HMC scope set (which is the base scope set for a dynamic scope set) in order to get its name.
                     *
                     * NOTE: Even though JSHint thinks that "scopeSetIndex" is an unused variable, removing it causes this
                     *       function to not work as intended.
                     */
                    $.each( scopeSets, function( scopeSetIndex, scopeSet ) {
                        if (pid !== null && pid == scopeSet.guid) {
                            hmcScopeSet = scopeSet;

                            logger.info("HMC scope set: " + scopeSet.name);

                            return false;
                        }
                    });

                    /*
                     * Find the associated for the target device type (HMC, AIX, Linux, or VIOS).
                     */
                    var matchingScopeSetName = hmcScopeSet.displayName;
                    if ((credentialOsName === "_AIX") || (credentialOsName === "_LINUX") || (credentialOsName === "_VIOS")) {
                        matchingScopeSetName = hmcScopeSet.displayName + credentialOsName;
                    }

                    $.each( scopeSets, $.proxy( function( scopeSetIndex, scopeSet ) {
                        if (    TV.typeIsString( typeof scopeSet.displayName ) &&
                                TV.typeIsString( typeof scopeSet.guid        ) &&
                                scopeSet.displayName === matchingScopeSetName ) {
                            logger.info( 'Matching scope set (' + 'index'       + ': ' + scopeSetIndex        + '; ' +
                                                                  'guid'        + ': ' + scopeSet.guid        + '; ' +
                                                                  'name'        + ': ' + scopeSet.name        + '; ' +
                                                                  'displayName' + ': ' + scopeSet.displayName + ').' );

                            nameForm.putHiddenInput("scopename", scopeSet.guid);
                            matchingScopeSet = scopeSet;

                            logger.info("Matching scope set: " + scopeSet.name);

                            return false;
                        }
                    }, this ) );
                }
            }
        } , this ) );

        // Retrieving Scope Sets from server...
        getScopeSetsCmd.execute();

        // Return the form...
        return firstForm;
    };


    /*
     * Set-up test credential button based on the type of credential being tested (HMC, AIX, Linux, VIOS).
     *
     * @param firstForm Form that will contain the test credential button.
     * @param actionAttribute Action taken when test credential button is clicked.
     *
     * @return Test credential button.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype.createTestButton = function(firstForm, actionAttribute) {
        var i18n = tsa.i18n;
        var testButton = ButtonFactory.buildButton( { type    : 'arrow',
                                                      label   : i18n.buttonTestCredential } );

        $(testButton).on('click', function( ) {
            tsa.clearSysMessages();

            var action = firstForm.getActionAttribute();  // Save current value and restore after panel is submitted.

            // Add flag to let completion of request know that this was a test and not a save request
            firstForm.putHiddenInput('testCredential', 'true');

            firstForm.setActionAttribute(actionAttribute);
            firstForm.submit();
            firstForm.setActionAttribute(action);

            $( firstForm ).on( 'executionFailed', function( event ) {
                if ( TV.typeIsObject( typeof event.response.payload.errors ) )  {
                    for ( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                        firstForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                    }
                    delete event.response.payload.errors;
                }
            } );
        } );

        return testButton;
    };



    //-------------------------------------------------------------------------

    /*==============================================================================*/
    /*==============================================================================*/
    /*===                                                                        ===*/
    /*=== GENERAL FUNCTIONS                                                      ===*/
    /*===                                                                        ===*/
    /*==============================================================================*/
    /*=============================================================================**/


    /**
     * Convert internal representation of authentication type to displayable text version.
     *
     * @param authtype Internal authentication type representation.
     *
     * @return Displayable string.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype.i18nAuthType = function ( authtype ) {
        var i18nLabel = '';

        if ( !TV.typeIsString( typeof authtype ) ) {
            authtype = '' + authtype;
        }

        switch( authtype ) {
            case 'authType_password' : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePassword  ; break;
            case 'authType_pki'      : i18nLabel = tsa.i18n.labelCredentialsAuthenticationTypePkiScoped ; break;
            default                  : i18nLabel = authtype;
        }

        return i18nLabel;
    };


    /**
     * Determine credential type (ie, HMC, AIX, Linux, or VIOS) based on internal name of a scope set.
     *
     * @param credentialName Internal name of credential.
     *
     * @return Target type of credential (ie, HMC, AIX, Linux, or VIOS).
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype.i18nCredentialType = function ( credentialName ) {
        var i18nLabel = '';

        if (credentialName.endsWith("_AIX_HMCCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoHMCCredsAixType;
        } else if (credentialName.endsWith("_LINUX_HMCCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoHMCCredsLinuxType;
        } else if (credentialName.endsWith("_VIOS_HMCCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoHMCCredsViosType;
        } else if (credentialName.endsWith("_HMCCrt_")) {
            i18nLabel = tsa.i18n.labelDiscoHMCCredsHmcType;
        }

        return i18nLabel;
    };


    /**
     * Convert hidden scope set name into displayable scope set name.
     *
     * @param hiddenScopeSetName Hidden scope set name.
     *
     * @return Displayable scope set name.
     */
    tsa.cmd.handlers.CredentialshmcHandler.prototype.getDisplayableScopeSetName = function ( hiddenScopeSetName ) {
        var displayScopeSetName = hiddenScopeSetName.replace("_AIX_HMCCrt_","");
        if (displayScopeSetName === hiddenScopeSetName) {
                displayScopeSetName = hiddenScopeSetName.replace("_LINUX_HMCCrt_","");
                if (displayScopeSetName === hiddenScopeSetName) {
                    displayScopeSetName = hiddenScopeSetName.replace("_VIOS_HMCCrt_","");
                    if (displayScopeSetName === hiddenScopeSetName) {
                        displayScopeSetName = hiddenScopeSetName.replace("_HMCCrt_","");
                    }
                }
            }

        return displayScopeSetName;
    };


    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.CredentialshmcHandler;
  }
);
