/*
 * IBM Technical Support Appliance Version 2 Release 8 Modification 0
 *
 * (C) Copyright IBM Corp. 2017, 2021
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/VerificationToolkit'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TablePanel'
  , 'tsa/cmd/ShowCommand'
  ],
  
  function( $
          , TV
          , VT
          , Command
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          , TablePanel
          , ShowCommand
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger('ImportvmwaredynamiclistHandler');
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler = function(parameters) {
        if (!TV.typeIsObject(typeof parameters) || (parameters === null)) parameters = /** @type {?} */ ( {} );
      
        // Calling the constructor of the super-class...
        AbstractHandler.prototype.constructor.call(this, parameters);
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ (tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype).constructor = tsa.cmd.handlers.ImportvmwaredynamiclistHandler;
    
    //-------------------------------------------------------------------------
    /**
     * The maximum recommended number of scope entries.
     * @private
     * @type {number}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.MAX_RECOMMENDED_ENTRIES = 400;
    
    /**
     * Form panel for choosing import list.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.formPanel = null;
    
    /**
     * Instructions message.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.instructionsMsg = null;
 
    /**
     * The HTML Input for the scope name.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.nameInput = null;
    
    /**
     * The HTML Input for the file containing scope entries.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.fileInput = null;
    
    /**
     * The map of file entries and their attributes.
     * @private
     * @type {Object.<string,{valid:boolean
     *                       ,count:number}>}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.fileEntries = null;

    /**
     * Name of the VMware Dynamic Scope Set.
     * @private
     * @type {string}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.scopeSetName = null;

    /**
     * PID of the Scope Set used to represent the VMweare instances.
     * @private
     * @type {string}
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.pid = null;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype._index = function() {
        /** @type {Object} */ var i18n = tsa.i18n;
      
        /** @type {tsa.widgets.ContentArea} */
        let importContentArea = new ContentArea ( { title   : i18n.labelVmwareImportScopeSetHeading,
                                                    helpDoc : 'sshelp_import_vmware_scope_set_panel.html' } );

        /** @type {tsa.widgets.TablePanel} */
        let tablePanel = new TablePanel( { title   : i18n.labelVmwareImportTableHeading,
                                           columns : [ { id : 'name'   , width : '80'}, { id : 'value' } ] } );

        /** @type {tsa.widgets.FormPanel} */
        this.formPanel = new FormPanel ( { title       : i18n.labelVmwareImportPaneHeading,
                                           description : i18n.labelVmwareImportScopeSetDesc } );

        /** @type {string} */ this.pid          = this.getParameter('pid');
        /** @type {string} */ this.scopeSetName = this.getParameter('scopeSetName');

      
        //
        // Create the scope set name pane
        //
        tablePanel.appendRow( [i18n.labelVmwareImportTableField, this.scopeSetName] );

        //
        // Create the Import pane
        //
        this.fileInput = $( '<input type="file" name="fileUpload" id="fileUpload" required/>' ).get(0);
      
        // Clear previous message
        $(this.fileInput).on('change', function() {tsa.clearSysMessages();});
      
        this.formPanel.appendFieldRow( { label    : i18n.labelVmwareImportScopeSet,
                                         required : true,
                                         element  : this.fileInput, 
                                         hint : i18n.labelVmwareImportScopeSetNameDesc} );

        /** @type {tsa.widgets.ButtonFactory} */ 
        let buttonContainer = ButtonFactory.buildButtonContainer();

        /** @type {Element} */
        let importButton = ButtonFactory.buildButton( { type    : 'arrow'
                                                      , label   : i18n.buttonVmwareImportScopeSet
                                                      , onclick : $.proxy( this.onImportButtonClicked, this ) } );
        /** @type {Element} */
        let cancelButton = ButtonFactory.buildButton( { type    : 'cancel'
                                                      , label   : i18n.buttonCancel } );

        /** @type {tsa.ShowCommand} */
        var cancelCommand = new ShowCommand( { action              : 'scopesetsvmware/show',
                                               clearSystemMessages : false,
                                               parameters          : { pid : this.pid } } ); 

        $(cancelButton).on('click', $.proxy(function( ) {
	        cancelCommand.execute();
        } ) );
      
        $(buttonContainer).append(importButton);
        $(buttonContainer).append(ButtonFactory.buildButtonSeparator);
        $(buttonContainer).append(cancelButton);
      
        this.instructionsMsg = importContentArea.appendMessage( { text : i18n.labelVmwareImportScopeSetCaption
                                                                      + '\n\n'
                                                                      + i18n.labelVmwareImportScopeSetLabelCaption } );

        importContentArea.append(tablePanel.getDomNode());
        importContentArea.append(this.formPanel.getDomNode());
        importContentArea.append(buttonContainer);
      
        this.show(importContentArea.getDomNode());
    };
    
    //-------------------------------------------------------------------------
    /**
     * Load file entries.
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.loadFileEntries = function() {

        this.fileEntries = {};
      
        if (!TV.typeIsUndefined(typeof FileReader          ) &&
            !TV.typeIsUndefined(typeof this.fileInput.files)) {
            if (this.fileInput.files.length > 0) {
                /** @type {Array.<string>} */ var valid = [];

                /** @type {number} */ var blankLines = 0;

                /** @type {FileReader} */ var fileReader = new FileReader();
          
                fileReader.onload = $.proxy(function(progressEvent) {
                    /** @type {Array.<string>} */ var lines      = fileReader.result.split( '\n' );
                    /** @type {number}         */ var blankLines = 0;
            
                    $.each(lines, $.proxy(function(lineIndex, line) {
                        if ((line = $.trim(line)).length > 0) {
                            valid.push(line);
                        } else {
                            blankLines++;
                        }
                    }
                    , this ) );
            
                    logger.info('File entries loaded. Results:'    + '\n' + 
                                '    Total lines: ' + lines.length + '\n' + 
                                '    Blank lines: ' + blankLines);
            
                    // Send list to TSA server to validate entries......
                    /** @type {tsa.cmd.Command} */ var cmd = new Command( { action     : 'importvmwaredynamiclist/ajaxValidateImportScopeSet',
                                                                            parameters : { 'pid'     : this.pid,
                                                                                           'entries' : valid.join( ' ' ) }
                                                                          } );
            
                    cmd.execute();    
            
                    $(cmd).on('executionSucceeded', $.proxy(function(event) {
            	        /** @type {string} */ 
                        var validEntries          = event.response.payload.validEntries;
                        /** @type {string} */
            	        var duplicateEntries      = event.response.payload.duplicateEntries;
                        /** @type {string} */
            	        var alreadyExistEntries   = event.response.payload.alreadyExistEntries;
                        /** @type {string} */
             	        var invalidEntries        = event.response.payload.invalidEntries;
                        /** @type {string} */
            	        var unresolvedFqdnEntries = event.response.payload.unresolvedFqdnEntries;

                        logger.info( 'File entries validated. Results:'                            + '\n' + 
                                     '    valid entries'              + ': ' + validEntries        + '\n' + 
                                     '    duplicate entries'          + ': ' + duplicateEntries    + '\n' + 
                                     '    already exist entries'      + ': ' + alreadyExistEntries + '\n' + 
                                     '    invalid entries'            + ': ' + invalidEntries      + '\n' + 
                                     '    unresolved FQDN entries'    + ': ' + unresolvedFqdnEntries );

                        // Show the confirmation dialog.  Simple dialog is displayed if all entries are valid, 
                        // otherwise a more in depth dialog is shown with errors in the list shown.
                        this.showConfirmationDialog(validEntries, 
                	                                duplicateEntries, 
                                                    alreadyExistEntries,
                		                            invalidEntries, 
                		                            unresolvedFqdnEntries);
            
                     }
                     , this ) );
          
                }
                , this );
          
                // Reading using default encoding: utf-8
                fileReader.readAsText( this.fileInput.files[0] );
        
            } else {
                logger.warn( 'File entries can\'t be loaded: file not selected.' );
            }
      
        } else {
            logger.error( 'Cannot read file: unsupported browser.' );
        }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Import Button: click event handler.
     * @param {Event} event Event
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.onImportButtonClicked = function( event ) {
        if (this.formPanel.validate()) {
            this.loadFileEntries();
        }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show confirmation dialog after TSA server analyzes the entries.
     * If problems were detected within the entry file, a new panel is shown with the errors encountered.
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.showConfirmationDialog = function(validEntries, 
                                                                                                duplicateEntries,
                                                                                                alreadyExistEntries, 
                                                                                                invalidEntries, 
                                                                                                unresolvedFqdnEntries) {
        /** @type {boolean} */ var tooMany   = false;
        /** @type {boolean} */ var noneValid = false;
        
        /** @type {Array.<string>} */ var valid        = [];
        /** @type {Array.<string>} */ var duplicate    = [];
        /** @type {Array.<string>} */ var alreadyExist = [];
        /** @type {Array.<string>} */ var invalid      = [];
        /** @type {Array.<string>} */ var unresolved   = [];

        // Turn entries lists into arrays if any entries exist.
        if (validEntries.length > 0) {
            valid = validEntries.split( ',' );
        } else {
	        noneValid = true;
        }
        
        if (duplicateEntries.length > 0) {
            duplicate = duplicateEntries.split( ',' );
        }
        if (alreadyExistEntries.length > 0) {
            alreadyExist = alreadyExistEntries.split( ',' );
        }
        if (invalidEntries.length > 0) {
        	invalid = invalidEntries.split( ',' );
        }
        if (unresolvedFqdnEntries.length > 0) {
            unresolved = unresolvedFqdnEntries.split( ',' );
        }
        
        if ((tooMany = (valid.length > tsa.cmd.handlers.ImportvmwaredynamiclistHandler.MAX_RECOMMENDED_ENTRIES)) ||
              (noneValid)               ||
              (invalid     .length > 0) ||
              (duplicate   .length > 0) ||
              (alreadyExist.length > 0) ||
              (unresolved  .length > 0)) {
	        /** @type {Object} */ var i18n = tsa.i18n;

	        /** @type {tsa.widgets.ContentArea} */
            let errorContentArea = new ContentArea ( { title   : i18n.labelVmwareImportScopeSetHeading } );  // This panel has no help text
	
            /** @type {Element} */ var problemsMsgNode = document.createElement('h3');
            /** @type {Element} */ var confirmMsgNode  = document.createElement('h3');

            /** @type {string}  */ var invalidMsgText      = tsa.i18n.labelImportScopeSetProblemInvalid;
            /** @type {string}  */ var duplicateMsgText    = tsa.i18n.labelImportScopeSetProblemValidDup;
            /** @type {string}  */ var alreadyExistMsgText = tsa.i18n.labelVmwareImportScopeSetProblemAlreadyExist
            /** @type {string}  */ var unresolvedMsgText   = tsa.i18n.labelImportScopeSetProblemResolve;
            /** @type {string}  */ var validMsgText        = tsa.i18n.labelImportScopeSetValidCount;
          
            /** @type {string}  */ var tooManyMsgText      = tsa.bind( tsa.i18n.labelVmwareImportScopeSetStatus, tsa.cmd.handlers.ImportvmwaredynamiclistHandler.MAX_RECOMMENDED_ENTRIES ) ;
            
            /** @type {Element} */ var buttonContainer     = ButtonFactory.buildButtonContainer();
            /** @type {string}  */ var bullet              = String.fromCharCode( 0x2022 );
          
            $(this.instructionsMsg)       .hide();
            $(this.formPanel.getDomNode()).hide();
          
            $.each(invalid     , function(i, entry) {invalidMsgText      += '\n' + bullet + ' ' + entry;} );
            $.each(duplicate   , function(i, entry) {duplicateMsgText    += '\n' + bullet + ' ' + entry;} );
            $.each(alreadyExist, function(i, entry) {alreadyExistMsgText += '\n' + bullet + ' ' + entry;} );
            $.each(unresolved  , function(i, entry) {unresolvedMsgText   += '\n' + bullet + ' ' + entry;} );
          
            validMsgText += ' ' + (valid.length);
          
            $(problemsMsgNode).text(tsa.i18n.labelVmwareImportScopeSetProblemsFound);
            errorContentArea.append(problemsMsgNode);

            if (tooMany) {
                $(confirmMsgNode).text(tsa.i18n.labelVmwareImportScopeSetConfirmLimit);
            } else {
	            if (!noneValid) {
		             let fileName = this.fileInput.files[0].name;

                     $(confirmMsgNode).text(tsa.bind(tsa.i18n.labelVmwareImportScopeSetConfirm, fileName));
                }
            }
          
            if (noneValid) { 
                errorContentArea.appendMessage( { level : 'ERROR', border : false, text : tsa.i18n.labelVmwareImportScopeSetProblemNoneValid } );
            }
            if (tooMany) { 
	            errorContentArea.appendMessage( { level : 'WARN' , border : false, text : tooManyMsgText } );
            }
            if (invalid.length > 0) {
	            errorContentArea.appendMessage( { level : 'WARN' , border : false, text : invalidMsgText } );
            }
            if (duplicate.length > 0) {
	            errorContentArea.appendMessage( { level : 'WARN' , border : false, text : duplicateMsgText } );
            }
            if (alreadyExist.length > 0) {
	            errorContentArea.appendMessage( { level : 'WARN' , border : false, text : alreadyExistMsgText } );
            }
            if (unresolved.length > 0) {
	            errorContentArea.appendMessage( { level : 'WARN' , border : false, text : unresolvedMsgText } );
            }

            /*
             * Return to show / edit of VMware Dynamic Scope Set regardless if user asks to import or cancel.
             */  
            var goBackOrCancelCommand = new ShowCommand( { action              : 'importvmwaredynamiclist/index',
                                                           clearSystemMessages : true,
                                                           parameters          : { pid          : this.pid,
                                                                                   scopeSetName : this.scopeSetName } } );
            
            /** @type {Element} */ var onAccept         = null; 
            /** @type {Element} */ var onGoBackOrCancel = null;
          
            onAccept = $.proxy( function( event ) {
                this.importScopeSet( valid );
            }, this );
          
            onGoBackOrCancel = $.proxy(function(event) {
                goBackOrCancelCommand.execute();
            }, this );
          
            if (!noneValid) {
                $(buttonContainer).append(ButtonFactory.buildButton( { type : 'arrow', label : tsa.i18n.buttonAccept, onclick : onAccept } ) );
                $(buttonContainer).append(ButtonFactory.buildButtonSeparator() );
         
                $(buttonContainer).append(ButtonFactory.buildButton( { type : 'cancel', label : tsa.i18n.buttonCancel, onclick : onGoBackOrCancel } ) );

                errorContentArea.append(confirmMsgNode);
                validMsgNode = errorContentArea.appendMessage( { level : 'INFO', border: false, text : validMsgText } );
            
            } else {
                $(buttonContainer).append(ButtonFactory.buildButton( { type : 'back', grayout : true, label : tsa.i18n.buttonGoBack, onclick : onGoBackOrCancel } ) );
                $(buttonContainer).append(ButtonFactory.buildButtonSeparator() );
            } 
          
            errorContentArea.append(buttonContainer);

            this.show(errorContentArea.getDomNode());
        
        } else {
            // A simple dialog is enough.  No problems found with import list.
            let fileName = this.fileInput.files[0].name;

            if (window.confirm(tsa.bind(tsa.i18n.labelVmwareImportScopeSetConfirm, fileName))) {
                this.importScopeSet( valid );
            }
        }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Import Scope Set: perform the server request.
     * @private
     * @param {Array.<string>} entries The entries, previously validated IPv4 addresses.
     */
    tsa.cmd.handlers.ImportvmwaredynamiclistHandler.prototype.importScopeSet = function( entries ) {
        /** @type {tsa.cmd.Command} */ var cmd = new Command( { action     : 'importvmwaredynamiclist/ajaxImportScopeSet'
                                                              , parameters : { 'name'    : this.scopeSetName
	                                                                         , 'pid'     : this.pid
                                                                             , 'entries' : entries.join( ',' ) }
                                                            } );
        $(cmd).on('executionSucceeded', $.proxy(function(event) {
             ( new ShowCommand( { action              : 'scopesetsvmware/show',
                                  clearSystemMessages : false,
                                  parameters          : { pid : this.pid }
                 } ) ).execute();       
        }, this ) );
      
        $(cmd).on('executionFailed', $.proxy(function(event) {
            if (TV.typeIsObject(typeof event.response.payload.errors)) {
                for( /** @type {string} */ var fieldName in event.response.payload.errors ) {
                    this.formPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
                }
            }
        }
        , this ) );
      
        cmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.ImportvmwaredynamiclistHandler;
  }
);