/*
 * IBM Technical Support Appliance Version 2 Release 8 Modification 0
 *
 * (C) Copyright IBM Corp. 2019, 2021
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/ShowCommand'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/LicenseFactory'
  ],
  
  function( $
          , TV
          , Command
          , ShowCommand
          , AbstractHandler
          , ContentArea
          , ButtonFactory
          , TablePanel
          , LicenseFactory
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'LicenseHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.LicenseHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.LicenseHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.LicenseHandler.prototype ).constructor = tsa.cmd.handlers.LicenseHandler;
    
    //-------------------------------------------------------------------------
    
    /**
     * Buffer of entries retrieved from server. Key: ID.
     * @private
     * @type {Object.<string,Object>}
     */
    tsa.cmd.handlers.LicenseHandler.entryBuffer = {};
    
    
    
    
    
    tsa.cmd.handlers.LicenseHandler.prototype._index = function(){
      /** @type {Object}                    */     var i18n                        = tsa.i18n;
      /** @type {tsa.widgets.ContentArea}   */     var contentArea                 = new ContentArea ( {    title         : i18n.labelLicenseHeading } );
      /** @type {element}                   */     var divBaseLicense              = null;
      /** @type {String}                    */     var separatelyLicenseText       = null;
      /** @type {String}                    */     var licenseTextInfo             = null;
      /** @type {String}                    */     var nonIBMNoticesText           = null;
      /** @type {element}                   */     var overLayWindowState          = LicenseFactory.buildOverLayWindows("stateWindow", "stateWindowsText", i18n.labelibmLicenseState);
      /** @type {element}                   */     var overLayWindowNotices        = LicenseFactory.buildOverLayWindows("noticeInfo", "noticeInfoText", i18n.labelibmLicenseNoticeHeading);
      /** @type {element}                   */     var overLayNonIBMNotices        = LicenseFactory.buildOverLayWindows("nonIBMNotices", "nonIBMNoticesText", i18n.labelibmLicenseNonIBMNoticeHeading);
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerStateofWork  = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerNoticeInfo   = ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerNonIBMNotices= ButtonFactory.buildButtonContainer();
      /** @type {tsa.widgets.ButtonFactory} */     var buttonContainerAccept       = ButtonFactory.buildButtonContainer();
      
      tsa.setLayout( 'wide' );
      
      //hide left menu
      $("#ibm-navigation").hide();
      
      /** @type {tsa.cmd.Command           */     var getLicenseData   = new Command({action:'license/getLicenseData'
                                                                                         ,clearSystemMessages:false
                                                                                          });
      
      /** @type {tsa.widgets.TablePanel} */      var tableNoticeInfo   = new TablePanel( { title    : i18n.labelibmLicenseNoticeHeading
                                                                                          , summary  : 'License statement info'
                                                                                          , columns  : [ { content : '' } ] } );
                                                                                        
      /** @type {tsa.widgets.TablePanel} */      var tableStatemAndWork = new TablePanel( { title    : i18n.labelibmLicenseHeading
                                                                                          , summary  : 'License statement work'
                                                                                          , columns  : [ { id : '' } ] } );
      
      /** @type {tsa.widgets.TablePanel} */      var tableNonIBMNotice = new TablePanel( { title    : i18n.labelibmLicenseNonIBMNoticeHeading
                                                                                          , summary  : 'License statement work'
                                                                                          , columns  : [ { id : '' } ] } );

      contentArea.appendMessage( { text : i18n.labelLicenseCaption } );
      
      $( buttonContainerStateofWork ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.labelibmLicenseStateButton
            , onclick : function( event )
              {
                $("#stateWindowsText").text(licenseTextInfo);
                $(overLayWindowState).show();
              }
            }
          )
        );
      $( buttonContainerNoticeInfo ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.labelibmLicenseNoticeButton
            , onclick : function( event )
              {
              $("#noticeInfoText").text(noticeInfoText);
              $(overLayWindowNotices).show();
              }
            }
          )
        );
      
      $( buttonContainerNonIBMNotices ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.labelibmLicenseNonIBMNoticeButton
            , onclick : function( event )
              {
              $("#nonIBMNoticesText").text(nonIBMNoticesText);
              $(overLayNonIBMNotices).show();
              }
            }
          )
        );
      
      tableStatemAndWork.appendRow([ buttonContainerStateofWork ]);
      tableNoticeInfo.appendRow([ buttonContainerNoticeInfo ]);
      tableNonIBMNotice.appendRow([ buttonContainerNonIBMNotices ]);

      $( buttonContainerAccept ).append(
          ButtonFactory.buildButton(
            { type    : 'arrow'
            , grayout : false
            , label   : i18n.buttonAccept
            , onclick : function( event )
              {
              /** @type {tsa.cmd.Command}           */ var acceptCommand = new Command( {  action : 'license/ajaxAccept'
                                                                            , clearSystemMessages : false } );
              acceptCommand.execute();
              
              $(acceptCommand).on('executionSucceeded', function(event)
              {
                $("#ibm-navigation").show();
	            ( new ShowCommand( { action : 'main/index' } ) ).execute(); 
              });
              }
            }
          )
        );
      
      $(getLicenseData).on('executionSucceeded', $.proxy(function(event) {
        noticeInfoText  = event.response.payload.noticesInfo;
        licenseTextInfo = event.response.payload.ibmLicenseText;
        nonIBMNoticesText  = event.response.payload.nonIBMNoticesText;
        
        divBaseLicense = LicenseFactory.buildBaseLicensePanel(event.response.payload.baseLicense);
        contentArea.append( divBaseLicense );
        contentArea.append( tableStatemAndWork.getDomNode() );
        contentArea.append( tableNonIBMNotice.getDomNode() );
        contentArea.append( tableNoticeInfo.getDomNode() );
        contentArea.append( buttonContainerAccept );
        contentArea.append( overLayWindowState );
        contentArea.append( overLayWindowNotices );
        contentArea.append( overLayNonIBMNotices );
       }, this));
      
      getLicenseData.execute();
      
      this.show( contentArea.getDomNode() );
      
    };
    
    //-------------------------------------------------------------------------
    return tsa.cmd.handlers.LicenseHandler;
  }
);