/*
 * IBM Technical Support Appliance Version 2 Release 8 Modification 0
 *
 * (C) Copyright IBM Corp. 2021
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'],
  
  function( $ )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'LicenseFactory' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  This class must not be instantiated. */
    
    tsa.widgets.LicenseFactory = function( parameters )
    {
      throw 'This class must not be instantiated.';
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Build OverLayWindows.
     * Parameters:
     *   idLayout    : Id for the OverLayWindow 
     *   idText      : Id for the License text element
     *   headingInfo : Header text data.
     *   
     * @param { id          : string
     *         , headingInfo : string
     *         , label   : string } parameters The parameters.
     * @return {Element}
     */
    tsa.widgets.LicenseFactory.buildOverLayWindows = function(idLayout,idText, headingInfo)
    {  
        /** @type {element} */ var divOverLay = $('<div class="ibm-common-overlay ibm-overlay-alt-two" style="display: block; top: 75px; left: -120.5px;" ></div>').get(0);
        $(divOverLay).attr("id", idLayout);
        $(divOverLay).append('<div class="ibm-head"><p><a class="ibm-common-overlay-close" href="#close">' + tsa.i18n.buttonOverlayClose + '</a></p></div>');
        /** @type {element} */ var aOverlayClose = $(divOverLay).find("a").get(0);
        $(divOverLay).append('<div class="ibm-body">'
                            + '<div class="ibm-main">'
                            + '<div id="licenseHeading" class="ibm-title">'
                            + ' <h2><div >' + headingInfo + '</div></h2>'
                            + '</div>'
                            + '<div id="ibmLicensePanelOverlay" class="ibm-container ibm-alternate">' 
                            + '    <div id="ibmLicense" class="ibm-container-body ibm-license-code-sample ibm-license-wordwrap" aria-label="' + tsa.i18n.labelseparatelyLicenseHeading + ' ">'
                            + '        <code >'
                            + '        <div id="' + idText + '"></div>'
                            + '        </code>'
                            + '    </div>'
                            + '</div>'
                            + '</div>'
                            + '</div>');
        $(divOverLay).append('<div class="ibm-footer"></div>');
        $(divOverLay).hide();
        $(aOverlayClose).on("click",function(){ 
             $("#"+idLayout).hide();
         });
        return divOverLay;
    };
    
    //-------------------------------------------------------------------------
    
    /**
     * Create Panel to display License text
     * Parameters:
     *   licenseText  : License Text 
     *   
     * @param { licenseText          : string } parameters The parameters.
     * @return {Element}
     */
    tsa.widgets.LicenseFactory.buildBaseLicensePanel = function(licenseText){
        /** @type {element} */ var divBaseLicense      = $('<div id="baseLicensePanel" class="ibm-container" style="display: block;"></div>').get(0);
        /** @type {element} */ var spanLicenseHeading  = $('<span id="baseLicenseDesc"></span>').get(0);
        /** @type {element} */ var aSpanLicenseHeading = document.createElement("a");
        /** @type {element} */ var hSpanLicenseHeading = document.createElement("h2");
        /** @type {element} */ var divLicense          = $('<div id="baseLicense" style="background:#ffffff" class="ibm-container-body ibm-license-code-sample ibm-license-wordwrap"></div>').get(0);
        /** @type {element} */ var code                = document.createElement("code");
        
        $(aSpanLicenseHeading).text(tsa.i18n.baseLicenseHeading);
        $(hSpanLicenseHeading).append(aSpanLicenseHeading);
        $(spanLicenseHeading).append(hSpanLicenseHeading);
        $(divBaseLicense).append(spanLicenseHeading);
        $(code).append(licenseText);
        $(divLicense).append(code);
        $(divBaseLicense).append(divLicense);
        
        return divBaseLicense;
    };
    //-------------------------------------------------------------------------
    
    return tsa.widgets.LicenseFactory;
  }
);